@@ -382,9 +382,10 @@ class IntegrationSigningTests(IntegrationTestHelpersMixin, TestCase):
         self.add_suite_to_archive(suite_name)
 
         workflow_template_result = Client.execute_command(
-            "create-workflow-template",
-            "update-suites",
+            "workflow-template",
+            "create",
             "update_suites",
+            "update-suites",
             stdin=yaml.safe_dump({}),
         )
         assert workflow_template_result.returncode == 0
