@@ -362,6 +362,7 @@ class IntegrationWorkflowDebianPipelineTests(
             "vendor": "debian",
             "codename": "trixie",
             "architectures": [self.architecture],
+            "arch_all_host_architecture": self.architecture,
             "sbuild_backend": BackendType.UNSHARE,
             "qa_suite": f"{suite_name}@{CollectionCategory.SUITE}",
             "autopkgtest_backend": BackendType.UNSHARE,
@@ -438,7 +439,9 @@ class IntegrationWorkflowDebianPipelineTests(
         )["workflow_id"]
         assert Client.wait_for_work_request_completed(workflow_id, "success")
 
-        work_requests = Client.execute_command("list-work-requests")
+        work_requests = Client.execute_command(
+            "list-work-requests", "--limit", "1000"
+        )
         update_suites_id = next(
             wr
             for wr in work_requests["results"]
