@@ -187,21 +187,21 @@ class DebusineHttpClient:
 
     @staticmethod
     def _handle_response_content(
-        response: requests.Response,
+        content: bytes,
         expected_class: type[ClientModel] | None,
         url: str,
     ) -> ClientModel | None:
         if expected_class is None:
-            if response.content != b"":
+            if content != b"":
                 raise exceptions.UnexpectedResponseError(
                     f"Server ({url}) expected to return an empty body. "
-                    f"Returned:\n{response.content!r}"
+                    f"Returned:\n{content!r}"
                 )
             else:
                 return None
         else:
             try:
-                return expected_class.parse_raw(response.content)
+                return expected_class.parse_raw(content)
             except ValidationError as exc:
                 raise exceptions.UnexpectedResponseError(
                     f"Server ({url}) did not return valid object. "
@@ -291,9 +291,11 @@ class DebusineHttpClient:
             ) from exc
 
         if error := self._debusine_problem(response):
-            raise exceptions.DebusineError(error)
+            raise exceptions.DebusineError(**error)
         elif response.status_code in expected_statuses:
-            return self._handle_response_content(response, expected_class, url)
+            return self._handle_response_content(
+                response.content, expected_class, url
+            )
         elif response.status_code == requests.codes.not_found:
             raise exceptions.NotFoundError(f'Not found ({url})')
         elif response.status_code == requests.codes.forbidden:
@@ -328,7 +330,7 @@ class DebusineHttpClient:
     @staticmethod
     def _debusine_problem(response: requests.Response) -> dict[Any, Any] | None:
         """If response is an application/problem+json returns the body JSON."""
-        if response.headers["content-type"] == "application/problem+json":
+        if response.headers.get("content-type") == "application/problem+json":
             try:
                 content = response.json()
             except json.JSONDecodeError:
