@@ -111,14 +111,11 @@ class ConfigHandlerTests(TestCase):
 
         server_config = self.valid_configuration()[f"server:{server_name}"]
 
-        self.assertEqual(
-            config._server_configuration(server_name),
-            {
-                'api-url': server_config["api-url"],
-                'scope': server_config["scope"],
-                'token': server_config["token"],
-            },
-        )
+        server = config._server_configuration(server_name)
+
+        self.assertEqual(server.api_url, server_config["api-url"])
+        self.assertEqual(server.scope, server_config["scope"])
+        self.assertEqual(server.api_token, server_config["token"])
 
     def test_server_configuration_default(self) -> None:
         """ConfigHandler.server_configuration() uses the default server."""
@@ -132,14 +129,10 @@ class ConfigHandlerTests(TestCase):
             f"server:{configuration['General']['default-server']}"
         ]
 
-        self.assertEqual(
-            config_handler.server_configuration(),
-            {
-                'api-url': default_server["api-url"],
-                'scope': default_server["scope"],
-                'token': default_server["token"],
-            },
-        )
+        server = config_handler.server_configuration()
+        self.assertEqual(server.api_url, default_server["api-url"])
+        self.assertEqual(server.scope, default_server["scope"])
+        self.assertEqual(server.api_token, default_server["token"])
 
     def test_server_configuration_kali(self) -> None:
         """ConfigHandler.server_configuration() return Kali."""
@@ -152,14 +145,10 @@ class ConfigHandlerTests(TestCase):
 
         config_server = config[f"server:{server_name}"]
 
-        self.assertEqual(
-            config_handler.server_configuration(),
-            {
-                'api-url': config_server["api-url"],
-                'scope': config_server["scope"],
-                'token': config_server["token"],
-            },
-        )
+        server = config_handler.server_configuration()
+        self.assertEqual(server.api_url, config_server["api-url"])
+        self.assertEqual(server.scope, config_server["scope"])
+        self.assertEqual(server.api_token, config_server["token"])
 
     def test_server_configuration_find_by_fqdn(self) -> None:
         """ConfigHandler.server_configuration() finds a section by FQDN."""
@@ -169,14 +158,10 @@ class ConfigHandlerTests(TestCase):
             config, server_name="debusine.kali.org/kali"
         )
         config_server = config["server:kali"]
-        self.assertEqual(
-            config_handler.server_configuration(),
-            {
-                "api-url": config_server["api-url"],
-                "scope": config_server["scope"],
-                "token": config_server["token"],
-            },
-        )
+        server = config_handler.server_configuration()
+        self.assertEqual(server.api_url, config_server["api-url"])
+        self.assertEqual(server.scope, config_server["scope"])
+        self.assertEqual(server.api_token, config_server["token"])
 
     def test_server_configuration_find_by_fqdn_wrong_scope(self) -> None:
         """ConfigHandler.server_configuration() requires FQDN/scope to match."""
