@@ -9,8 +9,9 @@
 
 """Unit tests for the Debusine class."""
 
+import base64
 import contextlib
-import datetime
+import datetime as dt
 import hashlib
 import io
 import json
@@ -54,6 +55,8 @@ from debusine.client.models import (
     AssetPermissionCheckResponse,
     AssetResponse,
     AssetsResponse,
+    CollectionData,
+    CollectionDataNew,
     CreateWorkflowRequest,
     FileRequest,
     FileResponse,
@@ -262,7 +265,7 @@ class SerializeLocalArtifactTests(TestCase):
 
     def test_with_expire_at(self) -> None:
         """``serialize_local_artifact`` includes ``expire_at``."""
-        expire_at = datetime.datetime.now() + datetime.timedelta(days=1)
+        expire_at = dt.datetime.now() + dt.timedelta(days=1)
 
         serialized = serialize_local_artifact(
             self.local_artifact, workspace="workspace", expire_at=expire_at
@@ -287,6 +290,33 @@ class DebusineTests(TestCase):
             self.api_url + "/", self.token, logger=self.logger
         )
 
+    def test_webui_url(self) -> None:
+        for api_url, scope, path, expected in (
+            (
+                "https://debusine.debian.org/api",
+                "scope",
+                "/a/b",
+                "https://debusine.debian.org/scope/a/b",
+            ),
+            (
+                "https://debusine.debian.org/subdir/api",
+                "scope",
+                "/a/b",
+                "https://debusine.debian.org/subdir/scope/a/b",
+            ),
+            (
+                "https://debusine.debian.org/",
+                "scope",
+                "/a/b",
+                "https://debusine.debian.org/scope/a/b",
+            ),
+        ):
+            with self.subTest(api_url=api_url, scope=scope, path=path):
+                debusine = Debusine(
+                    api_url, self.token, logger=self.logger, scope=scope
+                )
+                self.assertEqual(debusine.webui_url(path), expected)
+
     @responses.activate
     def test_work_request_iter(self) -> None:
         """Debusine.work_request_iter yields work requests."""
@@ -547,11 +577,11 @@ class DebusineTests(TestCase):
         )
         self.assert_token_key_included_in_all_requests('token')
 
-    def verify_workflow_template_create_success(
+    def verify_legacy_workflow_template_create_success(
         self, workspace: str | None = None
     ) -> None:
         """
-        Verify Debusine.workflow_template_create().
+        Verify Debusine.legacy_workflow_template_create().
 
         :param workspace: if it's None: assert that the client's request
           does not contain "workspace" (it's assigned by the server). The
@@ -590,14 +620,16 @@ class DebusineTests(TestCase):
             fields_to_post.add("workspace")
 
         # workflow_template_to_post can contain "workspace=None". The method
-        # Debusine.workflow_template_create will remove this key if it's
+        # Debusine.legacy_workflow_template_create will remove this key if it's
         # None.
         workflow_template_to_post = WorkflowTemplateRequest(
             **workflow_template_fields
         )
 
         actual_workflow_template_response = (
-            self.debusine.workflow_template_create(workflow_template_to_post)
+            self.debusine.legacy_workflow_template_create(
+                workflow_template_to_post
+            )
         )
         self.assertEqual(
             actual_workflow_template_response,
@@ -614,14 +646,14 @@ class DebusineTests(TestCase):
         )
 
     @responses.activate
-    def test_workflow_template_create_success(self) -> None:
+    def test_legacy_workflow_template_create_success(self) -> None:
         """Post a new workflow template."""
-        self.verify_workflow_template_create_success(workspace="Testing")
+        self.verify_legacy_workflow_template_create_success(workspace="Testing")
 
     @responses.activate
-    def test_workflow_template_create_no_workspace_success(self) -> None:
+    def test_legacy_workflow_template_create_no_workspace_success(self) -> None:
         """Post a new workflow template."""
-        self.verify_workflow_template_create_success(workspace=None)
+        self.verify_legacy_workflow_template_create_success(workspace=None)
 
     def verify_workflow_create_success(
         self, workspace: str | None = None
@@ -703,7 +735,7 @@ class DebusineTests(TestCase):
         workspace = "test"
         download_artifact_tar_gz_url = "https://example.com/path/"
         category = "Testing"
-        created_at = datetime.datetime.now(datetime.UTC)
+        created_at = dt.datetime.now(dt.UTC)
         data: dict[str, Any] = {}
 
         get_artifact_response = create_artifact_response(
@@ -744,13 +776,13 @@ class DebusineTests(TestCase):
 
         workspace = "test"
         work_request = 5
-        expire_at = datetime.datetime.now() + datetime.timedelta(days=1)
+        expire_at = dt.datetime.now() + dt.timedelta(days=1)
 
         artifact_response = create_artifact_response(
             id=2,
             workspace=workspace,
             category="Testing",
-            created_at=datetime.datetime.now(datetime.UTC),
+            created_at=dt.datetime.now(dt.UTC),
             data={},
             files_to_upload=list(files.keys()),
         )
@@ -1351,7 +1383,7 @@ class DebusineTests(TestCase):
         asset_data = SigningKeyData(
             purpose=KeyPurpose.OPENPGP_UPLOAD,
             fingerprint="ABC123",
-            public_key="PUBLIC KEY",
+            public_key=base64.b64encode(b"PUBLIC KEY").decode(),
             description="Description",
         )
         asset_request = {
@@ -1409,7 +1441,7 @@ class DebusineTests(TestCase):
         asset_data = SigningKeyData(
             purpose=KeyPurpose.OPENPGP_UPLOAD,
             fingerprint="ABC123",
-            public_key="PUBLIC KEY",
+            public_key=base64.b64encode(b"PUBLIC KEY").decode(),
             description="Description",
         )
         assets_response = AssetsResponse.parse_obj(
@@ -1491,8 +1523,8 @@ class DebusineTests(TestCase):
         responses.add(
             responses.POST,
             (
-                f"{self.debusine.api_url}/asset/debusine:signing-key/"
-                f"openpgp:ABC123/sign_with/"
+                f"{self.debusine.api_url}/asset/debusine%3Asigning-key/"
+                f"openpgp%3AABC123/sign_with/"
             ),
             json=permission_check_response.dict(),
         )
@@ -1783,6 +1815,98 @@ class DebusineTests(TestCase):
         self.assertEqual(res.chain, [])
         responses.assert_call_count(url, 1)
 
+    @responses.activate
+    def test_collection_get(self) -> None:
+        url = f"{self.debusine.api_url}/collection/System/debian%3Asuite/sid/"
+        result = CollectionData(
+            id=42,
+            name="sid",
+            category=CollectionCategory.SUITE,
+            full_history_retention_period=None,
+            metadata_only_retention_period=None,
+            data={"test": True},
+        )
+        responses.add(responses.GET, url, json=result.dict())
+        res = self.debusine.collection_get("System", "debian:suite", "sid")
+        self.assertEqual(res, result)
+        responses.assert_call_count(url, 1)
+
+    @responses.activate
+    def test_collection_get_timedelta(self) -> None:
+        url = f"{self.debusine.api_url}/collection/System/debian%3Asuite/sid/"
+        result = CollectionData(
+            id=42,
+            name="sid",
+            category=CollectionCategory.SUITE,
+            full_history_retention_period=3,
+            metadata_only_retention_period=4,
+            data={"test": True},
+        )
+        responses.add(responses.GET, url, json=result.dict())
+        res = self.debusine.collection_get(
+            "System", result.category, result.name
+        )
+        self.assertEqual(res, result)
+        responses.assert_call_count(url, 1)
+
+    @responses.activate
+    def test_collection_create(self) -> None:
+        url = f"{self.debusine.api_url}/collection/System/"
+        data = CollectionDataNew(
+            name="sid",
+            category=CollectionCategory.SUITE,
+            full_history_retention_period=7,
+            metadata_only_retention_period=14,
+            data={"test": 2},
+        )
+        result = CollectionData(id=41, **data.dict())
+        responses.add(
+            responses.POST,
+            url,
+            json=result.dict(),
+            status=requests.codes.created,
+        )
+        res = self.debusine.collection_create("System", data)
+        self.assertEqual(res, result)
+        responses.assert_call_count(url, 1)
+
+    @responses.activate
+    def test_collection_update(self) -> None:
+        url = f"{self.debusine.api_url}/collection/System/42/"
+        result = CollectionData(
+            id=42,
+            name="sid",
+            category=CollectionCategory.SUITE,
+            full_history_retention_period=7,
+            metadata_only_retention_period=14,
+            data={"test": 2},
+        )
+        responses.add(responses.PATCH, url, json=result.dict())
+        res = self.debusine.collection_update("System", result)
+        self.assertEqual(res, result)
+        responses.assert_call_count(url, 1)
+
+    @responses.activate
+    def test_collection_iter(self) -> None:
+        url = f"{self.debusine.api_url}/collection/System/"
+        data = CollectionData(
+            id=42,
+            name="sid",
+            category=CollectionCategory.SUITE,
+            full_history_retention_period=None,
+            metadata_only_retention_period=None,
+            data={"test": True},
+        )
+        listing_response = create_listing_response(data)
+        responses.add(
+            responses.GET,
+            url,
+            json=model_to_json_serializable_dict(listing_response),
+        )
+        res = list(self.debusine.collection_iter("System"))
+        self.assertEqual(res, [data])
+        responses.assert_call_count(url, 1)
+
 
 class DebusineAsyncTests(server.DebusineAioHTTPTestCase, TestCase):
     """
@@ -2093,7 +2217,7 @@ class DebusineAsyncTests(server.DebusineAioHTTPTestCase, TestCase):
         The file in last_completed_at existed and its contents is read.
         """
         command = ["this-is-the-command"]
-        last_completed_at = datetime.datetime.now(datetime.UTC).isoformat()
+        last_completed_at = dt.datetime.now(dt.UTC).isoformat()
 
         last_completed_at_file = self.create_temporary_file(
             contents=json.dumps(
