@@ -9,8 +9,9 @@
 
 """Test for the main entry point of debusine."""
 
+import base64
+import datetime as dt
 import secrets
-from datetime import datetime, timedelta, timezone
 from typing import Any, NotRequired, TypedDict
 
 try:
@@ -28,6 +29,7 @@ from debusine.client.models import (
     EnrollPayload,
     FileRequest,
     FilesRequestType,
+    Link,
     LookupMultipleResponse,
     LookupResultType,
     LookupSingleResponse,
@@ -36,6 +38,7 @@ from debusine.client.models import (
     RelationResponse,
     RelationType,
     RelationsResponse,
+    ServerObject,
     StrMaxLength255,
     StrictBaseModel,
     WorkspaceInheritanceChain,
@@ -49,6 +52,38 @@ from debusine.test.test_utils import (
 )
 
 
+class LinkTests(TestCase):
+    """Tests for :py:class:`Link`."""
+
+    def test_can_be_str(self) -> None:
+        for link, expected in (
+            (Link(href="foo"), True),
+            (Link(href="foo", type="bar"), False),
+        ):
+            with self.subTest(link=link.dict(exclude_unset=True)):
+                self.assertEqual(link.can_be_str(), expected)
+
+    def test_extra_arguments(self) -> None:
+        link = Link.parse_obj(
+            {"href": "foo", "rel": "bar", "title": "Baz", "type": "wibble"}
+        )
+        self.assertEqual(link.href, "foo")
+        self.assertEqual(link.type, "wibble")
+        self.assertFalse(hasattr(link, "rel"))
+
+
+class ServerObjectTests(TestCase):
+    """Tests for ServerObject."""
+
+    def test_extra_json_fields(self) -> None:
+        # Ensure client models ignore extra fields in serialized data, so that
+        # new ones can be added without breaking old clients
+        serialized = {"extra": 1}
+        o = ServerObject.parse_obj(serialized)
+        self.assertEqual(o.links, {})
+        self.assertFalse(hasattr(o, "extra"))
+
+
 class WorkRequestTests(TestCase):
     """Tests for WorkRequest."""
 
@@ -137,7 +172,7 @@ class ArtifactCreateRequestTests(TestCase):
             }
         )
         self.data: dict[str, Any] = {}
-        self.expire_at = datetime.now() + timedelta(days=1)
+        self.expire_at = dt.datetime.now() + dt.timedelta(days=1)
 
         self.artifact = ArtifactCreateRequest(
             category=self.category,
@@ -189,7 +224,7 @@ class ArtifactResponseTests(TestCase):
             id=self.artifact_id,
             category="Testing",
             workspace="some workspace",
-            created_at=datetime.now(timezone.utc),
+            created_at=dt.datetime.now(dt.timezone.utc),
             download_tar_gz_url="https://example.com/some/path/",
             files_to_upload=self.files_to_upload,
         )
@@ -316,7 +351,7 @@ class ModelToTest(pydantic.BaseModel):
     """Simple model to be used in a test."""
 
     id: int
-    created_at: datetime
+    created_at: dt.datetime
 
 
 class ModelToDictTests(TestCase):
@@ -325,7 +360,7 @@ class ModelToDictTests(TestCase):
     def test_model_to_json_serializable_dict(self) -> None:
         """model_to_json_serializable_dict return datetime as str."""
         obj_id = 4
-        created_at = datetime.now(tz=timezone.utc)
+        created_at = dt.datetime.now(tz=dt.timezone.utc)
 
         model = ModelToTest(id=obj_id, created_at=created_at)
         expected = {"id": obj_id, "created_at": created_at.isoformat()}
@@ -345,7 +380,7 @@ class AssetResponseTest(TestCase):
             "data": {
                 "purpose": "openpgp",
                 "fingerprint": "ABC123",
-                "public_key": "PUBLIC KEY",
+                "public_key": base64.b64encode(b"PUBLIC KEY").decode(),
                 "description": "Description",
             },
             "work_request": 5,
@@ -379,7 +414,7 @@ class AssetsResponsesTest(TestCase):
                     data=SigningKeyData(
                         purpose=KeyPurpose.OPENPGP_UPLOAD,
                         fingerprint="ABC123",
-                        public_key="PUBLIC KEY",
+                        public_key=base64.b64encode(b"PUBLIC KEY").decode(),
                         description="Description",
                     ),
                 ).dict()
@@ -616,7 +651,7 @@ class WorkspaceInheritanceChainTests(TestCase):
 
     def test_empty(self) -> None:
         val = WorkspaceInheritanceChain()
-        self.assertEqual(val.dict(), {"chain": []})
+        self.assertEqual(val, WorkspaceInheritanceChain(chain=[]))
 
     def test_from_strings(self) -> None:
         El = WorkspaceInheritanceChainElement
