@@ -9,7 +9,9 @@
 
 """Data models for assets."""
 
+import base64
 import enum
+from collections.abc import Generator, Iterable
 from typing import Any, Generic, Self, TYPE_CHECKING, TypeAlias, TypeVar, cast
 
 from django.core.exceptions import ImproperlyConfigured, ValidationError
@@ -202,7 +204,9 @@ class AssetUsageQuerySet(models.QuerySet["AssetUsage", A], Generic[A]):
         """Keep only resources where the user has the given role."""
         if not user or not user.is_authenticated:
             return self.none()
-        return self.filter(role.q(user)).distinct()
+        return self.filter(
+            pk__in=self.model.objects.filter(role.q(user)).distinct()
+        )
 
     @permission_filter(workers=Allow.PASS)
     def can_sign_with(self, user: PermissionUser) -> "AssetUsageQuerySet[A]":
@@ -245,6 +249,7 @@ class Asset(models.Model):
     objects = AssetManager.from_queryset(AssetQuerySet)()
 
     class Meta(TypedModelMeta):
+        base_manager_name = "objects"
         constraints = [
             JsonDataUniqueConstraint(
                 fields=["data->>'name'"],
@@ -404,6 +409,7 @@ class AssetUsage(models.Model):
     objects = AssetUsageManager.from_queryset(AssetUsageQuerySet)()
 
     class Meta(TypedModelMeta):
+        base_manager_name = "objects"
         constraints = [
             UniqueConstraint(
                 fields=["asset", "workspace"],
@@ -448,3 +454,11 @@ class AssetUsageRole(models.Model):
     )
 
     role = models.CharField(max_length=32, choices=AssetUsageRoles.choices)
+
+
+def get_public_keys(signing_keys: Iterable[Asset]) -> Generator[bytes]:
+    """Yield public keys from each of some signing-key assets."""
+    for signing_key in signing_keys:
+        signing_key_data = signing_key.data_model
+        assert isinstance(signing_key_data, SigningKeyData)
+        yield base64.b64decode(signing_key_data.public_key)
