@@ -9,8 +9,8 @@
 
 """Data models for db collections."""
 
+import datetime as dt
 import re
-from datetime import datetime
 from functools import cached_property
 from typing import (
     Any,
@@ -28,8 +28,17 @@ from django.contrib.postgres.constraints import ExclusionConstraint
 from django.contrib.postgres.fields import RangeOperators
 from django.core.exceptions import ValidationError
 from django.db import models
-from django.db.models import CheckConstraint, F, Q, QuerySet, UniqueConstraint
+from django.db.models import (
+    CheckConstraint,
+    F,
+    JSONField,
+    Q,
+    QuerySet,
+    UniqueConstraint,
+    Value,
+)
 from django.db.models.fields.json import KT
+from django.db.models.functions import Coalesce
 from django.urls import reverse
 from django.utils import timezone
 
@@ -53,12 +62,14 @@ from debusine.db.models.permissions import (
 from debusine.db.models.workspaces import Workspace
 
 if TYPE_CHECKING:
+    from django.http import HttpRequest
     from django_stubs_ext.db.models import TypedModelMeta
 
     from debusine.db.models.artifacts import Artifact
     from debusine.db.models.auth import User
     from debusine.db.models.work_requests import WorkRequest
     from debusine.server.collections import CollectionManagerInterface
+    from debusine.web.views.ui.collections import CollectionItemUI, CollectionUI
 else:
     TypedModelMeta = object
 
@@ -70,7 +81,7 @@ collection_name_regex = re.compile(r"^[A-Za-z][A-Za-z0-9+._-]*$")
 
 def is_valid_collection_name(value: str) -> bool:
     """Check if value is a valid scope name."""
-    return bool(collection_name_regex.match(value))
+    return value == "_" or bool(collection_name_regex.match(value))
 
 
 def validate_collection_name(value: str) -> None:
@@ -112,6 +123,18 @@ class CollectionQuerySet(QuerySet["Collection", A], Generic[A]):
         # Delegate to workspace can_configure check
         return self.filter(workspace__in=Workspace.objects.can_configure(user))
 
+    def exported_suites(self) -> "CollectionQuerySet[Any]":
+        """Filter to exported ``debian:suite`` collections."""
+        return (
+            self.filter(category=CollectionCategory.SUITE)
+            .annotate(
+                exported=Coalesce(
+                    "data__exported", Value("true"), output_field=JSONField()
+                )
+            )
+            .exclude(exported=False)
+        )
+
 
 class CollectionManager(models.Manager["Collection"]):
     """Manager for Collection model."""
@@ -176,6 +199,7 @@ class Collection(models.Model):
     data = models.JSONField(default=dict, blank=True)
 
     class Meta(TypedModelMeta):
+        base_manager_name = "objects"
         constraints = [
             UniqueConstraint(
                 fields=["name", "category", "workspace"],
@@ -208,6 +232,12 @@ class Collection(models.Model):
 
         return CollectionManagerInterface.get_manager_for(self)
 
+    def ui(self, request: "HttpRequest") -> "CollectionUI":
+        """Return a UI helper for this instance."""
+        from debusine.web.views.ui.collections import CollectionUI
+
+        return CollectionUI.get(request, self)
+
     def get_absolute_url(self) -> str:
         """Return the canonical URL to display the collection."""
         from debusine.server.scopes import urlconf_scope
@@ -270,7 +300,7 @@ class CollectionItemQuerySet(QuerySet["CollectionItem"]):
         """Filter to only active collection items."""
         return super().filter(removed_at__isnull=True)
 
-    def active_at(self, at: datetime) -> "CollectionItemQuerySet":
+    def active_at(self, at: dt.datetime) -> "CollectionItemQuerySet":
         """Filter to collection items that were active at a given time."""
         return self.filter(
             Q(created_at__lte=at),
@@ -305,7 +335,7 @@ class CollectionItemManager(models.Manager["CollectionItem"]):
         parent_collection: Collection,
         name: str,
         data: BaseArtifactDataModel | dict[str, Any],
-        created_at: datetime | None = None,
+        created_at: dt.datetime | None = None,
         created_by_user: "User",
         created_by_workflow: Optional["WorkRequest"],
         replaced_by: Optional["CollectionItem"] = None,
@@ -345,7 +375,7 @@ class CollectionItemManager(models.Manager["CollectionItem"]):
         parent_collection: Collection,
         name: str,
         data: dict[str, Any],
-        created_at: datetime | None = None,
+        created_at: dt.datetime | None = None,
         created_by_user: "User",
         created_by_workflow: Optional["WorkRequest"] = None,
         replaced_by: Optional["CollectionItem"] = None,
@@ -378,7 +408,7 @@ class CollectionItemManager(models.Manager["CollectionItem"]):
         parent_collection: Collection,
         name: str,
         data: dict[str, Any],
-        created_at: datetime | None = None,
+        created_at: dt.datetime | None = None,
         created_by_user: "User",
         created_by_workflow: Optional["WorkRequest"] = None,
         replaced_by: Optional["CollectionItem"] = None,
@@ -404,7 +434,7 @@ class CollectionItemManager(models.Manager["CollectionItem"]):
             **kwargs,
         )
 
-    def drop_full_history(self, at: datetime) -> None:
+    def drop_full_history(self, at: dt.datetime) -> None:
         """
         Drop artifacts from collections after full_history_retention_period.
 
@@ -417,7 +447,7 @@ class CollectionItemManager(models.Manager["CollectionItem"]):
             )
         ).update(artifact=None)
 
-    def drop_metadata(self, at: datetime) -> None:
+    def drop_metadata(self, at: dt.datetime) -> None:
         """
         Delete old collection items.
 
@@ -554,6 +584,12 @@ class CollectionItem(models.Model):
         """Format item name following item_template."""
         return name_template.format(**expanded_variables)
 
+    def ui(self, request: "HttpRequest") -> "CollectionItemUI":
+        """Return a UI helper for this instance."""
+        from debusine.web.views.ui.collections import CollectionItemUI
+
+        return CollectionItemUI.get(request, self)
+
     def get_absolute_url(self) -> str:
         """Return the canonical URL to display the item."""
         from debusine.server.scopes import urlconf_scope
@@ -590,6 +626,7 @@ class CollectionItem(models.Model):
         )
 
     class Meta(TypedModelMeta):
+        base_manager_name = "objects"
         constraints = [
             JsonDataUniqueConstraint(
                 fields=[
