@@ -100,7 +100,7 @@ class TaskDatabase(TaskDatabaseInterface):
                     workspace=self.work_request.workspace,
                     user=self.work_request.created_by,
                     default_category=default_category,
-                    workflow_root=self.work_request.get_workflow_root(),
+                    workflow_root=self.work_request.workflow_root,
                     expect_type=LookupChildType.ARTIFACT,
                 ).artifact
             )
@@ -136,7 +136,7 @@ class TaskDatabase(TaskDatabaseInterface):
                     workspace=self.work_request.workspace,
                     user=self.work_request.created_by,
                     default_category=default_category,
-                    workflow_root=self.work_request.get_workflow_root(),
+                    workflow_root=self.work_request.workflow_root,
                     expect_type=LookupChildType.ARTIFACT,
                 )
             ]
@@ -215,7 +215,7 @@ class TaskDatabase(TaskDatabaseInterface):
                     workspace=self.work_request.workspace,
                     user=self.work_request.created_by,
                     default_category=default_category,
-                    workflow_root=self.work_request.get_workflow_root(),
+                    workflow_root=self.work_request.workflow_root,
                     expect_type=LookupChildType.COLLECTION,
                 ).collection
             )
