@@ -17,6 +17,7 @@ from django.db import transaction
 
 from debusine.artifacts.models import TaskTypes
 from debusine.db.context import context
+from debusine.db.isolation import IsolationLevel, set_transaction
 from debusine.db.models import WorkRequest
 from debusine.db.models.work_requests import (
     compute_workflow_last_activity,
@@ -76,19 +77,37 @@ def _run_server_task_or_error(work_request: WorkRequest) -> bool:
             # No need to update DB state
             return False
         else:
-            work_request.mark_completed(
-                WorkRequest.Results.SUCCESS
-                if result
-                else WorkRequest.Results.FAILURE
-            )
-            return result
+            # Lock the whole workflow and mark the work request as
+            # completed.
+            with transaction.atomic():
+                WorkRequest.objects.filter(
+                    id=work_request.id
+                ).lock_workflows_for_update()
+                work_request.mark_completed(
+                    WorkRequest.Results.SUCCESS
+                    if result
+                    else WorkRequest.Results.FAILURE
+                )
+                return result
 
 
 # mypy complains that celery.shared_task is untyped, which is true, but we
 # can't fix that here.
 @shared_task  # type: ignore[misc]
+# Unlike some other Celery tasks, this must _not_ be decorated with
+# @transaction.atomic, because it needs to do finer-grained transaction
+# control.
 def run_server_task(work_request_id: int) -> bool:
-    """Run a :py:class:`BaseTask` via Celery."""
+    """
+    Run a :py:class:`BaseServerTask` via Celery.
+
+    :param work_request_id: The ID of the work request to run.
+    :raises WorkRequest.DoesNotExist: if ``work_request_id`` does not exist.
+    :raises WorkRequestNotPending: if the given work request's status is not
+      "pending".
+    :return: True if the server task succeeded; False if it failed or raised
+      an exception.
+    """
     try:
         work_request = WorkRequest.objects.get(pk=work_request_id)
     except WorkRequest.DoesNotExist:
@@ -103,6 +122,16 @@ def run_server_task(work_request_id: int) -> bool:
         )
         raise WorkRequestNotPending
 
+    # We would normally lock the whole workflow as close to the start of the
+    # operation as possible, holding that lock until the end of the
+    # transaction to minimize the risk of deadlocks when updating multiple
+    # work requests in succession.  However, server tasks may take a long
+    # time to run, and some of them manage their own transactions.
+    #
+    # WorkRequest.mark_running has no cascading effects on other work
+    # requests, and at this point we're in Django's default autocommit mode,
+    # so the worst case at this point is that it fails to obtain a lock on
+    # this work request; it cannot cause a deadlock.
     work_request.mark_running()
 
     context.reset()
@@ -111,20 +140,25 @@ def run_server_task(work_request_id: int) -> bool:
     try:
         return _run_server_task_or_error(work_request)
     except ServerTaskRunError as exc:
-        logger.error(
-            "Error running work request %s/%s (%s): %s",
-            work_request.task_type,
-            work_request.task_name,
-            work_request.id,
-            exc.message,
-        )
-        work_request.mark_completed(
-            WorkRequest.Results.ERROR,
-            output_data=OutputData(
-                errors=[OutputDataError(message=exc.message, code=exc.code)]
-            ),
-        )
-        raise
+        # Lock the whole workflow and record the error.
+        with transaction.atomic():
+            logger.error(
+                "Error running work request %s/%s (%s): %s",
+                work_request.task_type,
+                work_request.task_name,
+                work_request.id,
+                exc.message,
+            )
+            WorkRequest.objects.filter(
+                id=work_request.id
+            ).lock_workflows_for_update()
+            work_request.mark_completed(
+                WorkRequest.Results.ERROR,
+                output_data=OutputData(
+                    errors=[OutputDataError(message=exc.message, code=exc.code)]
+                ),
+            )
+            return False
 
 
 # mypy complains that celery.shared_task is untyped, which is true, but we
@@ -138,15 +172,13 @@ def update_workflows() -> None:
     The ``workflow_last_activity_at`` and ``workflow_runtime_status`` fields
     of workflows are used in the web UI and need to be kept up to date.
     """
+    # We may encounter serialization failures, but if so it's probably OK to
+    # just let the next run of this task catch up.
+    set_transaction(IsolationLevel.REPEATABLE_READ)
     need_update = WorkRequest.objects.filter(
         workflows_need_update=True
-    ).select_for_update()
-    workflow_ancestor_ids = workflow_ancestors(need_update).values_list(
-        "id", flat=True
-    )
-    for workflow in WorkRequest.objects.filter(
-        id__in=workflow_ancestor_ids
-    ).select_for_update():
+    ).lock_workflows_for_update()
+    for workflow in workflow_ancestors(need_update):
         assert workflow.task_type == TaskTypes.WORKFLOW
         workflow.workflow_last_activity_at = compute_workflow_last_activity(
             workflow
