@@ -9,9 +9,9 @@
 
 """The collection manager for debusine:task-configuration collections."""
 
+import datetime as dt
 import itertools
 from collections.abc import Generator, Iterable
-from datetime import datetime
 from typing import Any
 
 from django.db import IntegrityError
@@ -48,7 +48,7 @@ class DebusineTaskConfigurationManager(CollectionManagerInterface):
         data: dict[str, Any] | None = None,
         name: str | None = None,  # noqa: U100
         replace: bool = False,
-        created_at: datetime | None = None,
+        created_at: dt.datetime | None = None,
         replaced_by: CollectionItem | None = None,
     ) -> CollectionItem:
         """Add bare data into the managed collection."""
@@ -243,3 +243,6 @@ def apply_configuration(
 
     # Apply overrides
     task_data.update(override_values)
+
+    # Store the ID of the task configuration collection that was used
+    task_data["task_configuration"] = config_collection.id
