@@ -9,7 +9,7 @@
 
 """Unit tests for DebianArchiveManager."""
 
-from datetime import datetime
+import datetime as dt
 from pathlib import PurePath
 from typing import ClassVar
 
@@ -331,7 +331,7 @@ class DebianArchiveManagerTests(TestCase):
 
         self.assertEqual(collection_item.artifact, readme_artifact)
         self.assertEqual(collection_item.removed_by_user, self.scenario.user)
-        self.assertIsInstance(collection_item.removed_at, datetime)
+        self.assertIsInstance(collection_item.removed_at, dt.datetime)
 
     def test_do_remove_item_collection(self) -> None:
         """``do_remove_item`` can remove a suite from the archive."""
@@ -342,7 +342,7 @@ class DebianArchiveManagerTests(TestCase):
 
         self.assertEqual(suite_item.collection, suite)
         self.assertEqual(suite_item.removed_by_user, self.scenario.user)
-        self.assertIsInstance(suite_item.removed_at, datetime)
+        self.assertIsInstance(suite_item.removed_at, dt.datetime)
 
     def test_do_remove_item_collection_retains_suite_constraints(self) -> None:
         """If `may_reuse_versions=False`, versions may not be reused."""
