@@ -9,10 +9,10 @@
 
 """Implementation of external file backends."""
 
+import datetime as dt
 import tempfile
 from collections.abc import Callable, Generator
 from contextlib import contextmanager
-from datetime import datetime
 from pathlib import Path
 from typing import Any, Self
 
@@ -175,7 +175,7 @@ class ExternalDebianSuiteFileBackend(
 
     def list_entries(
         self,
-        mtime_filter: Callable[[datetime], bool] | None = None,  # noqa: U100
+        mtime_filter: Callable[[dt.datetime], bool] | None = None,  # noqa: U100
     ) -> Generator[ExternalDebianSuiteFileBackendEntry, None, None]:
         """Yield entries in this backend."""
         # We cannot currently iterate over all files in an external suite
