@@ -8,7 +8,7 @@
 # contained in the LICENSE file.
 
 """Tests for the management command manage_worker."""
-import datetime
+import datetime as dt
 from typing import ClassVar
 
 from django.utils import timezone
@@ -64,8 +64,8 @@ class ManageWorkerCommandTests(TestCase):
         )
         work_request_finished = self.playground.create_worker_task(
             worker=self.worker,
-            status=WorkRequest.Statuses.COMPLETED,
             started_at=timezone.now(),
+            result=WorkRequest.Results.SUCCESS,
         )
 
         self.assertTrue(self.token.enabled)
@@ -97,6 +97,4 @@ class ManageWorkerCommandTests(TestCase):
             work_request_finished.status, WorkRequest.Statuses.COMPLETED
         )
         self.assertEqual(work_request_finished.worker, self.worker)
-        self.assertIsInstance(
-            work_request_finished.started_at, datetime.datetime
-        )
+        self.assertIsInstance(work_request_finished.started_at, dt.datetime)
