@@ -9,7 +9,7 @@
 
 """Tests for the management command manage_worker."""
 
-import datetime
+import datetime as dt
 from typing import Any, ClassVar
 from unittest.mock import patch
 
@@ -126,8 +126,8 @@ class WorkerCommandTests(TabularOutputTests, TestCase):
         )
         work_request_finished = self.playground.create_worker_task(
             worker=self.worker,
-            status=WorkRequest.Statuses.COMPLETED,
             started_at=timezone.now(),
+            result=WorkRequest.Results.SUCCESS,
         )
 
         self.assertTrue(self.token.enabled)
@@ -154,9 +154,7 @@ class WorkerCommandTests(TabularOutputTests, TestCase):
             work_request_finished.status, WorkRequest.Statuses.COMPLETED
         )
         self.assertEqual(work_request_finished.worker, self.worker)
-        self.assertIsInstance(
-            work_request_finished.started_at, datetime.datetime
-        )
+        self.assertIsInstance(work_request_finished.started_at, dt.datetime)
 
     def test_edit_metadata(self) -> None:
         """Set the worker metadata from the YAML file."""
@@ -315,12 +313,12 @@ class WorkerCommandTests(TabularOutputTests, TestCase):
 
         def mock_edit(self_: YamlEditor[dict[str, Any]]) -> bool:
             self_.value = {"distributions": ["buster", "bullseye"]}
+            self_.cleanup()
             return True
 
         with patch(
             "debusine.utils.input.YamlEditor.edit",
-            side_effect=mock_edit,
-            autospec=True,
+            new=mock_edit,
         ):
             stdout, stderr, exit_code = call_command(
                 "worker",
@@ -344,7 +342,11 @@ class WorkerCommandTests(TabularOutputTests, TestCase):
         self.worker.static_metadata = {"distributions": ["potato"]}
         self.worker.save()
 
-        with patch("debusine.utils.input.YamlEditor.edit", return_value=False):
+        def fake_edit(self_: YamlEditor[dict[str, Any]]) -> bool:
+            self_.cleanup()
+            return False
+
+        with patch("debusine.utils.input.YamlEditor.edit", new=fake_edit):
             call_command(
                 "worker",
                 "edit_metadata",
