@@ -15,8 +15,8 @@ from debusine.server.tasks import ServerNoop
 from debusine.tasks import TaskConfigError
 
 
-class ServerNoopTaskTests(TestCase):
-    """Test the servernoop task."""
+class ServerNoopTests(TestCase):
+    """Test the ServerNoop task."""
 
     def test_execute_result_true(self) -> None:
         """If result=True, the task returns True."""
