@@ -10,14 +10,15 @@
 """Unit tests for Celery integration."""
 
 import logging
+import threading
 from typing import Any
 from unittest import mock
 
 from django.db import connections, transaction
 
 from debusine.db.models import WorkRequest, Worker
+from debusine.db.models.work_requests import workflow_ancestors
 from debusine.server.celery import (
-    ServerTaskRunError,
     WorkRequestNotPending,
     run_server_task,
     update_workflows,
@@ -83,8 +84,8 @@ class RunServerTaskTests(TestCase):
             level=logging.ERROR,
         ):
             result = run_server_task.apply(args=(work_request_id,))
-        self.assertTrue(result.failed())
-        self.assertIsInstance(result.result, WorkRequest.DoesNotExist)
+        with self.assertRaises(WorkRequest.DoesNotExist):
+            result.get()
 
     def test_not_pending(self) -> None:
         """The Celery task fails if the work request is not pending."""
@@ -97,11 +98,11 @@ class RunServerTaskTests(TestCase):
             level=logging.ERROR,
         ):
             result = run_server_task.apply(args=(work_request.id,))
-        self.assertTrue(result.failed())
-        self.assertIsInstance(result.result, WorkRequestNotPending)
+        with self.assertRaises(WorkRequestNotPending):
+            result.get()
 
     def test_bad_task_name(self) -> None:
-        """The Celery task fails if the work request has a bad task name."""
+        """The Celery task logs but does not fail if given a bad task name."""
         work_request = self.create_assigned_work_request()
         work_request.task_name = "nonexistent"
         work_request.save()
@@ -116,10 +117,7 @@ class RunServerTaskTests(TestCase):
             transaction.atomic(),
         ):
             result = run_server_task.apply(args=(work_request.id,))
-        self.assertTrue(result.failed())
-        self.assertIsInstance(result.result, ServerTaskRunError)
-        self.assertEqual(result.result.message, expected_message)
-        self.assertEqual(result.result.code, "setup-failed")
+        self.assertFalse(result.get())
         work_request.refresh_from_db()
         self.assertIsNotNone(work_request.started_at)
         self.assertIsNotNone(work_request.completed_at)
@@ -137,7 +135,7 @@ class RunServerTaskTests(TestCase):
         )
 
     def test_bad_task_data(self) -> None:
-        """The Celery task fails if the work request has bad task data."""
+        """The Celery task logs but does not fail if given bad task data."""
         work_request = self.create_assigned_work_request()
         work_request.task_data = {"nonexistent": True}
         work_request.configured_task_data = {"nonexistent": True}
@@ -155,10 +153,7 @@ class RunServerTaskTests(TestCase):
             transaction.atomic(),
         ):
             result = run_server_task.apply(args=(work_request.id,))
-        self.assertTrue(result.failed())
-        self.assertIsInstance(result.result, ServerTaskRunError)
-        self.assertTrue(result.result.message.startswith(expected_message))
-        self.assertEqual(result.result.code, "configure-failed")
+        self.assertFalse(result.get())
         work_request.refresh_from_db()
         self.assertIsNotNone(work_request.started_at)
         self.assertIsNotNone(work_request.completed_at)
@@ -178,7 +173,7 @@ class RunServerTaskTests(TestCase):
         )
 
     def test_external_task(self) -> None:
-        """The Celery task fails if asked to run an external task."""
+        """The Celery task logs but does not fail on an external task."""
         work_request = self.playground.create_worker_task(
             task_name="noop", task_data={"result": True}, worker=self.worker
         )
@@ -193,10 +188,7 @@ class RunServerTaskTests(TestCase):
             transaction.atomic(),
         ):
             result = run_server_task.apply(args=(work_request.id,))
-        self.assertTrue(result.failed())
-        self.assertIsInstance(result.result, ServerTaskRunError)
-        self.assertEqual(result.result.message, expected_message)
-        self.assertEqual(result.result.code, "wrong-task-type")
+        self.assertFalse(result.get())
         work_request.refresh_from_db()
         self.assertIsNotNone(work_request.started_at)
         self.assertIsNotNone(work_request.completed_at)
@@ -214,7 +206,7 @@ class RunServerTaskTests(TestCase):
         )
 
     def test_task_raises_exception(self) -> None:
-        """The Celery task fails if the task raises an exception."""
+        """The Celery task logs but does not fail if the task raises."""
         work_request = self.create_assigned_work_request(
             task_data=ServerNoopData(exception=True),
         )
@@ -229,13 +221,7 @@ class RunServerTaskTests(TestCase):
             transaction.atomic(),
         ):
             result = run_server_task.apply(args=(work_request.id,))
-        self.assertTrue(result.failed())
-        self.assertIsInstance(result.result, ServerTaskRunError)
-        self.assertEqual(
-            result.result.message,
-            "Execution failed: Client requested an exception",
-        )
-        self.assertEqual(result.result.code, "execute-failed")
+        self.assertFalse(result.get())
         work_request.refresh_from_db()
         self.assertIsNotNone(work_request.started_at)
         self.assertIsNotNone(work_request.completed_at)
@@ -268,8 +254,7 @@ class RunServerTaskTests(TestCase):
             transaction.atomic(),
         ):
             result = run_server_task.apply(args=(work_request.id,))
-        self.assertFalse(result.failed())
-        self.assertFalse(result.result)
+        self.assertFalse(result.get())
         work_request.refresh_from_db()
         self.assertIsNotNone(work_request.started_at)
         self.assertIsNone(work_request.completed_at)
@@ -284,8 +269,7 @@ class RunServerTaskTests(TestCase):
         )
         with transaction.atomic():
             result = run_server_task.apply(args=(work_request.id,))
-        self.assertFalse(result.failed())
-        self.assertFalse(result.result)
+        self.assertFalse(result.get())
         work_request.refresh_from_db()
         self.assertIsNotNone(work_request.started_at)
         self.assertIsNotNone(work_request.completed_at)
@@ -300,8 +284,7 @@ class RunServerTaskTests(TestCase):
         )
         with transaction.atomic():
             result = run_server_task.apply(args=(work_request.id,))
-        self.assertFalse(result.failed())
-        self.assertTrue(result.result)
+        self.assertTrue(result.get())
         work_request.refresh_from_db()
         self.assertIsNotNone(work_request.started_at)
         self.assertIsNotNone(work_request.completed_at)
@@ -322,8 +305,7 @@ class RunServerTaskTransactionTests(TransactionTestCase):
         )
         work_request.assign_worker(worker)
         result = run_server_task.apply(args=(work_request.id,))
-        self.assertFalse(result.failed())
-        self.assertTrue(result.result)
+        self.assertTrue(result.get())
 
     def test_task_managing_transactions(self) -> None:
         """An atomic block is not used for tasks that manage transactions."""
@@ -334,8 +316,53 @@ class RunServerTaskTransactionTests(TransactionTestCase):
         )
         work_request.assign_worker(worker)
         result = run_server_task.apply(args=(work_request.id,))
-        self.assertFalse(result.failed())
-        self.assertTrue(result.result)
+        self.assertTrue(result.get())
+
+    def test_locks_workflow_before_completion(self) -> None:
+        workflow = self.playground.create_workflow()
+        server_noop = workflow.create_child_server(
+            task_name="servernoop", task_data=ServerNoopData(result=True)
+        )
+        noop = workflow.create_child_worker(task_name="noop")
+        self.playground.advance_work_request(server_noop, mark_pending=True)
+        self.playground.assign_worker(server_noop)
+        unrelated = self.playground.create_worker_task()
+
+        with mock.patch.object(
+            WorkRequest,
+            "mark_completed",
+            side_effect=(
+                lambda *args, **kwargs: self.assert_work_requests_locked(
+                    locked=[workflow, server_noop, noop], unlocked=[unrelated]
+                )
+            ),
+        ):
+            result = run_server_task.apply(args=(server_noop.id,))
+
+        self.assertTrue(result.get())
+
+    def test_locks_workflow_before_completion_error(self) -> None:
+        workflow = self.playground.create_workflow()
+        server_noop = workflow.create_child_server(
+            task_name="servernoop", task_data=ServerNoopData(exception=True)
+        )
+        noop = workflow.create_child_worker(task_name="noop")
+        self.playground.advance_work_request(server_noop, mark_pending=True)
+        self.playground.assign_worker(server_noop)
+        unrelated = self.playground.create_worker_task()
+
+        with mock.patch.object(
+            WorkRequest,
+            "mark_completed",
+            side_effect=(
+                lambda *args, **kwargs: self.assert_work_requests_locked(
+                    locked=[workflow, server_noop, noop], unlocked=[unrelated]
+                )
+            ),
+        ):
+            result = run_server_task.apply(args=(server_noop.id,))
+
+        self.assertFalse(result.get())
 
 
 class UpdateWorkflowTransactionTests(TransactionTestCase):
@@ -343,23 +370,87 @@ class UpdateWorkflowTransactionTests(TransactionTestCase):
 
     def test_updates_workflows(self) -> None:
         """The Celery task updates workflow properties."""
-        with mock.patch("django.db.transaction.on_commit"):
+        with (
+            transaction.atomic(),
+            mock.patch("django.db.transaction.on_commit"),
+        ):
             workflow = self.playground.create_workflow()
             work_request = workflow.create_child_worker("noop")
-            self.assertTrue(work_request.mark_aborted())
+            self.assertFalse(work_request.workflows_need_update)
+            work_request.mark_aborted()
             self.assertTrue(work_request.workflows_need_update)
+            unrelated = self.playground.create_worker_task()
 
-        result = update_workflows.apply()
-        self.assertFalse(result.failed())
-        self.assertIsNone(result.result)
+        with (
+            transaction.atomic(),
+            mock.patch("debusine.server.celery.set_transaction"),
+        ):
+            result = update_workflows.apply()
+            self.assertIsNone(result.get())
 
-        workflow.refresh_from_db()
-        work_request.refresh_from_db()
+            workflow.refresh_from_db()
+            work_request.refresh_from_db()
+            self.assertEqual(
+                workflow.workflow_last_activity_at, work_request.completed_at
+            )
+            self.assertEqual(
+                workflow.workflow_runtime_status,
+                WorkRequest.RuntimeStatuses.ABORTED,
+            )
+            self.assertFalse(work_request.workflows_need_update)
+            self.assert_work_requests_locked(
+                locked=[workflow, work_request], unlocked=[unrelated]
+            )
+
+    def test_repeatable_read(self) -> None:
+        """The task runs in the correct transaction isolation level."""
+        with (
+            transaction.atomic(),
+            mock.patch("django.db.transaction.on_commit"),
+        ):
+            workflows = [self.playground.create_workflow() for _ in range(2)]
+            work_requests = [
+                workflow.create_child_worker("noop") for workflow in workflows
+            ]
+            self.assertFalse(work_requests[0].workflows_need_update)
+            work_requests[0].mark_aborted()
+            self.assertTrue(work_requests[0].workflows_need_update)
+            self.assertFalse(work_requests[1].workflows_need_update)
+
+        def create_workflow_needs_update() -> None:
+            with transaction.atomic():
+                with mock.patch("django.db.transaction.on_commit"):
+                    work_requests[1].mark_aborted()
+                self.assertTrue(work_requests[1].workflows_need_update)
+
+        def create_workflow_needs_update_in_thread(
+            *args: Any, **kwargs: Any
+        ) -> Any:
+            thread = threading.Thread(target=create_workflow_needs_update)
+            thread.start()
+            thread.join()
+            return mock.DEFAULT
+
+        with mock.patch(
+            "debusine.server.celery.workflow_ancestors",
+            side_effect=create_workflow_needs_update_in_thread,
+            wraps=workflow_ancestors,
+        ):
+            result = update_workflows.apply()
+            self.assertIsNone(result.get())
+
+        # The first workflow is updated.
+        workflows[0].refresh_from_db()
+        work_requests[0].refresh_from_db()
         self.assertEqual(
-            workflow.workflow_last_activity_at, work_request.completed_at
+            workflows[0].workflow_last_activity_at,
+            work_requests[0].completed_at,
         )
-        self.assertEqual(
-            workflow.workflow_runtime_status,
-            WorkRequest.RuntimeStatuses.ABORTED,
-        )
-        self.assertFalse(work_request.workflows_need_update)
+        self.assertFalse(work_requests[0].workflows_need_update)
+
+        # The second workflow is untouched, since it didn't need an update
+        # until after our task's transaction started.
+        workflows[1].refresh_from_db()
+        work_requests[1].refresh_from_db()
+        self.assertIsNone(workflows[1].workflow_last_activity_at)
+        self.assertTrue(work_requests[1].workflows_need_update)
