@@ -27,6 +27,7 @@ from rest_framework import status
 from rest_framework.test import APIClient
 
 from debusine.artifacts.models import ArtifactCategory
+from debusine.client.models import Link
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
@@ -43,7 +44,8 @@ from debusine.db.tests.utils import _calculate_hash_from_data
 from debusine.server.scopes import urlconf_scope
 from debusine.server.serializers import ArtifactSerializerResponse
 from debusine.server.views.artifacts import UploadFileView
-from debusine.test.django import TestCase, TestResponseType
+from debusine.server.views.tests.base import TestCase
+from debusine.test.django import TestResponseType
 from debusine.test.test_utils import data_generator
 
 
@@ -876,6 +878,23 @@ class ArtifactViewTests(TestCase):
         data = self.assertAPIResponseOk(response)
         self.assertEqual(data, expected)
 
+    def test_debusine_artifact_get(self) -> None:
+        artifact, _ = self.playground.create_artifact()
+        debusine = self.get_debusine()
+        result = debusine.artifact_get(artifact.pk)
+        self.assertEqual(result.id, artifact.id)
+        self.assertEqual(result.workspace, artifact.workspace.name)
+        self.assertEqual(
+            result.links["webui_self"], artifact.get_absolute_url()
+        )
+        self.assertEqual(
+            result.links["download_tar_gz"],
+            Link(
+                href=artifact.get_absolute_url_download() + "?archive=tar.gz",
+                type="application/tar+gzip",
+            ),
+        )
+
 
 class UploadFileViewTests(TestCase):
     """Tests for UploadFileView class."""
