@@ -11,7 +11,10 @@
 
 import datetime as dt
 from typing import Any, ClassVar, assert_never
+from unittest import mock
 
+from django.db import IntegrityError
+from django.db.models import Max
 from django.urls import reverse
 from rest_framework import status
 
@@ -21,6 +24,8 @@ from debusine.artifacts.models import (
     DebusineTaskConfiguration,
     TaskTypes,
 )
+from debusine.client.exceptions import DebusineError
+from debusine.client.models import CollectionData, CollectionDataNew
 from debusine.db.models import Collection, CollectionItem, Workspace
 from debusine.db.playground import scenarios
 from debusine.server.collections import DebusineTaskConfigurationManager
@@ -28,10 +33,10 @@ from debusine.server.views.collections import (
     ComparableItem,
     TaskConfigurationCollectionView,
 )
+from debusine.server.views.tests.base import TestCase
 from debusine.test.django import (
     AllowAll,
     DenyAll,
-    TestCase,
     TestResponseType,
     override_permission,
 )
@@ -613,105 +618,193 @@ class TaskConfigurationCollectionViewTests(TestCase):
                 )
 
 
-class CollectionViewTests(TestCase):
-    """Tests for CollectionView."""
+class CollectionViewSetTests(TestCase):
+    """Tests for :py:class:`CollectionViewSet`."""
 
     scenario = scenarios.DefaultContextAPI()
-    collection: ClassVar[Collection]
-    manager: ClassVar[DebusineTaskConfigurationManager]
+    sid: ClassVar[Collection]
+    sid_links: ClassVar[dict[str, str]]
+    trixie: ClassVar[Collection]
 
     @classmethod
     def setUpTestData(cls) -> None:
         """Set up common test data."""
         super().setUpTestData()
-        cls.collection = cls.playground.create_collection(
-            "test", CollectionCategory.TEST, data={"key": "value"}
+        cls.sid = cls.playground.create_collection(
+            "sid", CollectionCategory.SUITE, data={"test": 1}
+        )
+        cls.trixie = cls.playground.create_collection(
+            "trixie", CollectionCategory.SUITE, data={"test": 1}
+        )
+        cls.sid_links = cls.get_links(cls.sid)
+
+    @classmethod
+    def get_links(cls, collection: Collection) -> dict[str, str]:
+        return {
+            "self": reverse(
+                "api:collection-detail",
+                kwargs={
+                    "workspace": collection.workspace.name,
+                    "pk": collection.pk,
+                },
+            ),
+            "webui_self": collection.get_absolute_url(),
+        }
+
+    def _workspace_url_kwarg(self, workspace: Workspace | str | None) -> str:
+        match workspace:
+            case None:
+                return self.scenario.workspace.name
+            case str():
+                return workspace
+            case _:
+                return workspace.name
+
+    def _reverse_list(
+        self,
+        workspace: Workspace | str | None,
+        **kwargs: str,
+    ) -> str:
+        """Build the URL for a test request."""
+        kwargs["workspace"] = self._workspace_url_kwarg(workspace)
+        return reverse("api:collection-list", kwargs=kwargs)
+
+    def _reverse_lookup(
+        self,
+        workspace: Workspace | str | None,
+        **kwargs: str,
+    ) -> str:
+        """Build the URL for a test request."""
+        kwargs["workspace"] = self._workspace_url_kwarg(workspace)
+        return reverse("api:collection-lookup", kwargs=kwargs)
+
+    def _reverse_detail(
+        self,
+        collection: int | Collection | None,
+        workspace: Workspace | str | None,
+        **kwargs: str,
+    ) -> str:
+        """Build the URL for a test request."""
+        kwargs["workspace"] = self._workspace_url_kwarg(workspace)
+
+        match collection:
+            case None:
+                kwargs["pk"] = str(self.sid.pk)
+            case int():
+                kwargs["pk"] = str(collection)
+            case Collection():
+                kwargs["pk"] = str(collection.pk)
+            case _ as unreachable:
+                assert_never(unreachable)
+
+        return reverse("api:collection-detail", kwargs=kwargs)
+
+    def _headers(self, user_token: bool) -> dict[str, str]:
+        """Build headers for a test request."""
+        match user_token:
+            case True:
+                return {"token": self.scenario.user_token.key}
+            case False:
+                return {}
+                pass
+            case _ as unreachable:
+                assert_never(unreachable)
+
+    def lookup(
+        self,
+        category: str = CollectionCategory.SUITE,
+        name: str = "sid",
+        workspace: Workspace | str | None = None,
+        user_token: bool = True,
+    ) -> TestResponseType:
+        """GET request for the lookup endpoint."""
+        return self.client.get(
+            self._reverse_lookup(workspace, category=category, name=name),
+            content_type="application/json",
+            headers=self._headers(user_token),
+        )
+
+    def list(
+        self,
+        workspace: Workspace | str | None = None,
+        user_token: bool = True,
+    ) -> TestResponseType:
+        """GET request for a named collection."""
+        return self.client.get(
+            self._reverse_list(workspace),
+            content_type="application/json",
+            headers=self._headers(user_token),
         )
 
     def get(
         self,
-        name: str = "test",
+        collection: int | Collection | None = None,
         workspace: Workspace | str | None = None,
-        category: CollectionCategory = CollectionCategory.TEST,
         user_token: bool = True,
     ) -> TestResponseType:
         """GET request for a named collection."""
-        match workspace:
-            case None:
-                workspace_name = self.scenario.workspace.name
-            case str():
-                workspace_name = workspace
-            case _:
-                workspace_name = workspace.name
-
-        match user_token:
-            case True:
-                headers = {"token": self.scenario.user_token.key}
-            case False:
-                headers = {}
-                pass
-            case _ as unreachable:
-                assert_never(unreachable)
-
         return self.client.get(
-            reverse(
-                "api:collection",
-                kwargs={
-                    "workspace": workspace_name,
-                    "ccat": category,
-                    "cname": name,
-                },
-            ),
+            self._reverse_detail(collection, workspace),
             content_type="application/json",
-            headers=headers,
+            headers=self._headers(user_token),
         )
 
     def post(
         self,
-        name: str = "test",
         workspace: Workspace | str | None = None,
-        category: CollectionCategory = CollectionCategory.TEST,
         user_token: bool = True,
         data: dict[str, Any] | None = None,
     ) -> TestResponseType:
-        """POST request for a named collection."""
-        match workspace:
-            case None:
-                workspace_name = self.scenario.workspace.name
-            case str():
-                workspace_name = workspace
-            case _:
-                workspace_name = workspace.name
-
-        match user_token:
-            case True:
-                headers = {"token": self.scenario.user_token.key}
-            case False:
-                headers = {}
-                pass
-            case _ as unreachable:
-                assert_never(unreachable)
-
-        if data is None:
-            data = {}
-
+        """POST request."""
         return self.client.post(
-            reverse(
-                "api:collection",
-                kwargs={
-                    "workspace": workspace_name,
-                    "ccat": category,
-                    "cname": name,
-                },
-            ),
-            data=data,
+            self._reverse_list(workspace),
+            data=data or {},
             content_type="application/json",
-            headers=headers,
+            headers=self._headers(user_token),
         )
 
+    def put(
+        self,
+        collection: int | Collection | None = None,
+        workspace: Workspace | str | None = None,
+        user_token: bool = True,
+        data: dict[str, Any] | None = None,
+    ) -> TestResponseType:
+        """PUT request for a named collection."""
+        return self.client.put(
+            self._reverse_detail(collection, workspace),
+            data=data or {},
+            content_type="application/json",
+            headers=self._headers(user_token),
+        )
+
+    def patch(
+        self,
+        collection: int | Collection | None = None,
+        workspace: Workspace | str | None = None,
+        user_token: bool = True,
+        data: dict[str, Any] | None = None,
+    ) -> TestResponseType:
+        """PATCH request for a named collection."""
+        return self.client.patch(
+            self._reverse_detail(collection, workspace),
+            data=data or {},
+            content_type="application/json",
+            headers=self._headers(user_token),
+        )
+
+    def assertSidUnchanged(self) -> None:
+        """Ensure self.sid is unchanged."""
+        self.sid.refresh_from_db()
+        self.assertEqual(self.sid.name, "sid")
+        self.assertEqual(self.sid.category, CollectionCategory.SUITE)
+        self.assertIsNone(self.sid.full_history_retention_period)
+        self.assertIsNone(self.sid.metadata_only_retention_period)
+        self.assertEqual(self.sid.data, {"test": 1})
+
     def test_unauthenticated(self) -> None:
         """Authentication is required."""
-        for method in "get", "post":
+        for method in ("lookup", "list", "post", "put", "patch"):
             with self.subTest(method=method):
                 response = getattr(self, method)(user_token=False)
                 self.assertResponseProblem(
@@ -725,7 +818,7 @@ class CollectionViewTests(TestCase):
 
     def test_workspace_not_found(self) -> None:
         """Workspace must exist."""
-        for method in "get", "post":
+        for method in ("lookup", "list", "post", "put", "patch"):
             with self.subTest(method=method):
                 response = getattr(self, method)(workspace="does-not-exist")
                 self.assertResponseProblem(
@@ -740,7 +833,7 @@ class CollectionViewTests(TestCase):
     def test_workspace_not_accessible(self) -> None:
         """User must be able to display the workspace."""
         workspace = self.playground.create_workspace(name="private")
-        for method in "get", "post":
+        for method in ("lookup", "list", "post", "put", "patch"):
             with self.subTest(method=method):
                 response = getattr(self, method)(workspace=workspace)
                 self.assertResponseProblem(
@@ -754,113 +847,222 @@ class CollectionViewTests(TestCase):
 
     def test_not_found(self) -> None:
         """The collection must exist."""
-        for method in "get", "post":
+        max_id = Collection.objects.aggregate(Max('id'))['id__max']
+        for method in ("get", "put", "patch"):
             with self.subTest(method=method):
-                response = getattr(self, method)(name="does-not-exist")
+                response = getattr(self, method)(collection=max_id + 1)
                 self.assertResponseProblem(
                     response,
-                    "Collection not found",
-                    detail_pattern=(
-                        "'debusine:test' collection 'does-not-exist'"
-                        " does not exist in workspace 'System'"
-                    ),
+                    "Object not found",
                     status_code=status.HTTP_404_NOT_FOUND,
                 )
 
-    def test_get_unauthorized(self) -> None:
+    def test_detail_unauthorized(self) -> None:
         """The user must be able to display the collection."""
-        with override_permission(Collection, "can_display", DenyAll):
-            response = self.get()
+        for method in ("get", "put", "patch"):
+            with (
+                self.subTest(method=method),
+                override_permission(Collection, "can_display", DenyAll),
+            ):
+                response = getattr(self, method)()
+                self.assertResponseProblem(
+                    response,
+                    "Object not found",
+                    status_code=status.HTTP_404_NOT_FOUND,
+                )
+
+    def test_create_unauthorized(self) -> None:
+        """The user must be able to create the collection."""
+        with override_permission(Workspace, "can_create_collections", DenyAll):
+            response = getattr(self, "post")()
             self.assertResponseProblem(
                 response,
-                ("playground cannot display collection test@debusine:test"),
+                "playground cannot create collections"
+                f" in {self.scenario.workspace}",
                 status_code=status.HTTP_403_FORBIDDEN,
             )
 
-    def test_post_unauthorized(self) -> None:
-        """The user must be able to display the collection."""
-        with override_permission(
-            Workspace, "can_edit_task_configuration", DenyAll
-        ):
-            response = self.post()
-            self.assertResponseProblem(
-                response,
-                "playground cannot configure collection test@debusine:test",
-                status_code=status.HTTP_403_FORBIDDEN,
-            )
+    def test_update_unauthorized(self) -> None:
+        """The user must be able to update the collection."""
+        for method in ("put", "patch"):
+            with (
+                self.subTest(method=method),
+                override_permission(Collection, "can_configure", DenyAll),
+            ):
+                response = getattr(self, method)()
+                self.assertResponseProblem(
+                    response,
+                    f"playground cannot configure collection {self.sid}",
+                    status_code=status.HTTP_403_FORBIDDEN,
+                )
 
-    def test_get(self) -> None:
+    def test_lookup(self) -> None:
+        response = self.lookup()
+        data = self.assertAPIResponseOk(response)
+        assert data == {
+            "id": self.sid.pk,
+            "name": self.sid.name,
+            "category": self.sid.category,
+            "full_history_retention_period": None,
+            "metadata_only_retention_period": None,
+            "data": {"test": 1},
+            "links": self.sid_links,
+        }
+
+    def test_lookup_not_found(self) -> None:
+        response = self.lookup(category=CollectionCategory.SUITE, name="buster")
+        self.assertResponseProblem(
+            response,
+            "Object not found",
+            status_code=status.HTTP_404_NOT_FOUND,
+        )
+
+    def test_detail(self) -> None:
         response = self.get()
         data = self.assertAPIResponseOk(response)
         assert data == {
-            "id": self.collection.pk,
-            "name": "test",
+            "id": self.sid.pk,
+            "name": self.sid.name,
+            "category": self.sid.category,
             "full_history_retention_period": None,
             "metadata_only_retention_period": None,
-            "data": {"key": "value"},
+            "data": {"test": 1},
+            "links": self.sid_links,
         }
 
-    @override_permission(Collection, "can_configure", AllowAll)
-    def test_post_noop(self) -> None:
-        response = self.post(data={})
-        new_data = self.assertAPIResponseOk(response)
-        assert new_data == {
-            "id": self.collection.pk,
-            "name": "test",
-            "full_history_retention_period": None,
-            "metadata_only_retention_period": None,
-            "data": {"key": "value"},
-        }
-        self.collection.refresh_from_db()
-        self.assertEqual(self.collection.name, "test")
-        self.assertIsNone(self.collection.full_history_retention_period)
-        self.assertIsNone(self.collection.metadata_only_retention_period)
-        self.assertEqual(self.collection.data, {"key": "value"})
-
-    @override_permission(Collection, "can_configure", AllowAll)
-    def test_post_noop_all_set(self) -> None:
+    @override_permission(Workspace, "can_create_collections", AllowAll)
+    def test_post(self) -> None:
         response = self.post(
             data={
-                "name": "test",
+                "category": CollectionCategory.SUITE,
+                "name": "new",
                 "full_history_retention_period": None,
                 "metadata_only_retention_period": None,
-                "data": {"key": "value"},
+                "data": {"created": True},
             }
         )
+        new_data = self.assertAPIResponseOk(
+            response, status_code=status.HTTP_201_CREATED
+        )
+        new = Collection.objects.get(
+            workspace=self.scenario.workspace,
+            category=CollectionCategory.SUITE,
+            name="new",
+        )
+        self.assertIsNone(new.full_history_retention_period)
+        self.assertIsNone(new.metadata_only_retention_period)
+        self.assertEqual(new.data, {"created": True})
+        assert new_data == {
+            "id": new.pk,
+            "category": CollectionCategory.SUITE,
+            "name": "new",
+            "full_history_retention_period": None,
+            "metadata_only_retention_period": None,
+            "data": {"created": True},
+            "links": self.get_links(new),
+        }
+
+    @override_permission(Workspace, "can_create_collections", AllowAll)
+    def test_post_name_conflict(self) -> None:
+        response = self.post(
+            data={
+                "category": CollectionCategory.SUITE,
+                "name": "sid",
+                "full_history_retention_period": None,
+                "metadata_only_retention_period": None,
+                "data": {},
+            }
+        )
+        self.assertResponseProblem(
+            response,
+            "Cannot create collection",
+            status_code=status.HTTP_400_BAD_REQUEST,
+            detail_pattern="A collection with the same name already exists",
+        )
+        self.assertSidUnchanged()
+
+    @override_permission(Workspace, "can_create_collections", AllowAll)
+    def test_post_unexpected_error(self) -> None:
+        with mock.patch(
+            "debusine.server.serializers.CollectionSerializer.save",
+            side_effect=IntegrityError("unexpected message"),
+        ):
+            response = self.post(
+                data={
+                    "category": CollectionCategory.SUITE,
+                    "name": "sid",
+                    "full_history_retention_period": None,
+                    "metadata_only_retention_period": None,
+                    "data": {},
+                }
+            )
+        self.assertResponseProblem(
+            response,
+            "Cannot create collection",
+            status_code=status.HTTP_400_BAD_REQUEST,
+        )
+        self.assertSidUnchanged()
+
+    @override_permission(Collection, "can_configure", AllowAll)
+    def test_patch_noop(self) -> None:
+        response = self.patch(data={})
+        new_data = self.assertAPIResponseOk(response)
+        assert new_data == {
+            "id": self.sid.pk,
+            "category": self.sid.category,
+            "name": self.sid.name,
+            "full_history_retention_period": None,
+            "metadata_only_retention_period": None,
+            "data": self.sid.data,
+            "links": self.sid_links,
+        }
+        self.assertSidUnchanged()
+
+    @override_permission(Collection, "can_configure", AllowAll)
+    def test_patch_noop_all_set(self) -> None:
+        response = self.patch(
+            data={
+                "name": "sid",
+                "full_history_retention_period": None,
+                "metadata_only_retention_period": None,
+                "data": {"test": 1},
+            },
+        )
         new_data = self.assertAPIResponseOk(response)
         assert new_data == {
-            "id": self.collection.pk,
-            "name": "test",
+            "id": self.sid.pk,
+            "category": self.sid.category,
+            "name": self.sid.name,
             "full_history_retention_period": None,
             "metadata_only_retention_period": None,
-            "data": {"key": "value"},
+            "data": self.sid.data,
+            "links": self.sid_links,
         }
-        self.collection.refresh_from_db()
-        self.assertEqual(self.collection.name, "test")
-        self.assertIsNone(self.collection.full_history_retention_period)
-        self.assertIsNone(self.collection.metadata_only_retention_period)
-        self.assertEqual(self.collection.data, {"key": "value"})
+        self.assertSidUnchanged()
 
     @override_permission(Collection, "can_configure", AllowAll)
-    def test_post_rename(self) -> None:
-        response = self.post(data={"name": "renamed"})
+    def test_patch_rename(self) -> None:
+        response = self.patch(data={"name": "renamed"})
         new_data = self.assertAPIResponseOk(response)
+        self.sid.refresh_from_db()
         assert new_data == {
-            "id": self.collection.pk,
+            "id": self.sid.pk,
+            "category": self.sid.category,
             "name": "renamed",
             "full_history_retention_period": None,
             "metadata_only_retention_period": None,
-            "data": {"key": "value"},
+            "data": self.sid.data,
+            "links": self.get_links(self.sid),
         }
-        self.collection.refresh_from_db()
-        self.assertEqual(self.collection.name, "renamed")
-        self.assertIsNone(self.collection.full_history_retention_period)
-        self.assertIsNone(self.collection.metadata_only_retention_period)
-        self.assertEqual(self.collection.data, {"key": "value"})
+        self.assertEqual(self.sid.name, "renamed")
+        self.assertEqual(self.sid.category, CollectionCategory.SUITE)
+        self.assertIsNone(self.sid.full_history_retention_period)
+        self.assertIsNone(self.sid.metadata_only_retention_period)
+        self.assertEqual(self.sid.data, {"test": 1})
 
     @override_permission(Collection, "can_configure", AllowAll)
-    def test_post_change_retention_periods(self) -> None:
-        response = self.post(
+    def test_patch_change_retention_periods(self) -> None:
+        response = self.patch(
             data={
                 "full_history_retention_period": 123,
                 "metadata_only_retention_period": 456,
@@ -868,44 +1070,50 @@ class CollectionViewTests(TestCase):
         )
         new_data = self.assertAPIResponseOk(response)
         assert new_data == {
-            "id": self.collection.pk,
-            "name": "test",
-            "full_history_retention_period": "00:02:03",
-            "metadata_only_retention_period": "00:07:36",
-            "data": {"key": "value"},
+            "id": self.sid.pk,
+            "category": self.sid.category,
+            "name": self.sid.name,
+            "full_history_retention_period": 123,
+            "metadata_only_retention_period": 456,
+            "data": self.sid.data,
+            "links": self.sid_links,
         }
-        self.collection.refresh_from_db()
-        self.assertEqual(self.collection.name, "test")
+        self.sid.refresh_from_db()
+        self.assertEqual(self.sid.name, "sid")
+        self.assertEqual(self.sid.category, CollectionCategory.SUITE)
         self.assertEqual(
-            self.collection.full_history_retention_period,
-            dt.timedelta(seconds=123),
+            self.sid.full_history_retention_period,
+            dt.timedelta(days=123),
         )
         self.assertEqual(
-            self.collection.metadata_only_retention_period,
-            dt.timedelta(seconds=456),
+            self.sid.metadata_only_retention_period,
+            dt.timedelta(days=456),
         )
-        self.assertEqual(self.collection.data, {"key": "value"})
+        self.assertEqual(self.sid.data, {"test": 1})
 
     @override_permission(Collection, "can_configure", AllowAll)
-    def test_post_change_data(self) -> None:
-        response = self.post(data={"data": {"changed": True}})
+    def test_patch_change_data(self) -> None:
+        response = self.patch(data={"data": {"changed": True}})
         new_data = self.assertAPIResponseOk(response)
         assert new_data == {
-            "id": self.collection.pk,
-            "name": "test",
+            "id": self.sid.pk,
+            "category": self.sid.category,
+            "name": self.sid.name,
             "full_history_retention_period": None,
             "metadata_only_retention_period": None,
             "data": {"changed": True},
+            "links": self.sid_links,
         }
-        self.collection.refresh_from_db()
-        self.assertEqual(self.collection.name, "test")
-        self.assertIsNone(self.collection.full_history_retention_period)
-        self.assertIsNone(self.collection.metadata_only_retention_period)
-        self.assertEqual(self.collection.data, {"changed": True})
+        self.sid.refresh_from_db()
+        self.assertEqual(self.sid.name, "sid")
+        self.assertEqual(self.sid.category, CollectionCategory.SUITE)
+        self.assertIsNone(self.sid.full_history_retention_period)
+        self.assertIsNone(self.sid.metadata_only_retention_period)
+        self.assertEqual(self.sid.data, {"changed": True})
 
     @override_permission(Collection, "can_configure", AllowAll)
-    def test_post_change_all(self) -> None:
-        response = self.post(
+    def test_patch_change_all(self) -> None:
+        response = self.patch(
             data={
                 "name": "renamed",
                 "full_history_retention_period": 123,
@@ -914,55 +1122,98 @@ class CollectionViewTests(TestCase):
             }
         )
         new_data = self.assertAPIResponseOk(response)
+        self.sid.refresh_from_db()
         assert new_data == {
-            "id": self.collection.pk,
+            "id": self.sid.pk,
             "name": "renamed",
-            "full_history_retention_period": "00:02:03",
-            "metadata_only_retention_period": "00:07:36",
+            "category": self.sid.category,
+            "full_history_retention_period": 123,
+            "metadata_only_retention_period": 456,
             "data": {"changed": True},
+            "links": self.get_links(self.sid),
         }
-        self.collection.refresh_from_db()
-        self.assertEqual(self.collection.name, "renamed")
+        self.assertEqual(self.sid.name, "renamed")
         self.assertEqual(
-            self.collection.full_history_retention_period,
-            dt.timedelta(seconds=123),
+            self.sid.full_history_retention_period,
+            dt.timedelta(days=123),
         )
         self.assertEqual(
-            self.collection.metadata_only_retention_period,
-            dt.timedelta(seconds=456),
+            self.sid.metadata_only_retention_period,
+            dt.timedelta(days=456),
         )
-        self.assertEqual(self.collection.data, {"changed": True})
+        self.assertEqual(self.sid.data, {"changed": True})
 
     @override_permission(Collection, "can_configure", AllowAll)
-    def test_post_rename_conflict(self) -> None:
-        self.playground.create_collection("renamed", CollectionCategory.TEST)
+    def test_patch_cannot_change_category(self) -> None:
+        response = self.patch(
+            data={
+                "name": "renamed",
+                "category": "debusine:test",
+                "full_history_retention_period": 123,
+                "metadata_only_retention_period": 456,
+                "data": {"changed": True},
+            }
+        )
+        self.assertResponseProblem(
+            response,
+            "The category of a collection cannot be changed",
+            status_code=status.HTTP_400_BAD_REQUEST,
+        )
+        self.assertSidUnchanged()
 
-        response = self.post(data={"name": "renamed"})
+    @override_permission(Collection, "can_configure", AllowAll)
+    def test_patch_rename_conflict(self) -> None:
+        self.playground.create_collection("renamed", CollectionCategory.SUITE)
+
+        response = self.patch(data={"name": "renamed"})
         self.assertResponseProblem(
             response,
             "New name conflicts with an existing collection",
             detail_pattern=(
-                "A collection with category 'debusine:test'"
+                "A collection with category 'debian:suite'"
                 " called 'renamed' already exists"
             ),
             status_code=status.HTTP_400_BAD_REQUEST,
         )
-        self.collection.refresh_from_db()
-        self.assertEqual(self.collection.name, "test")
+        self.assertSidUnchanged()
 
     @override_permission(Collection, "can_configure", AllowAll)
-    def test_post_rename_singleton_ok(self) -> None:
+    def test_patch_rename_no_conflict_in_other_workspaces(self) -> None:
+        other = self.playground.create_workspace(name="other")
+        self.playground.create_collection(
+            "renamed", CollectionCategory.SUITE, workspace=other
+        )
+
+        response = self.patch(data={"name": "renamed"})
+        new_data = self.assertAPIResponseOk(response)
+        self.sid.refresh_from_db()
+        assert new_data == {
+            "id": self.sid.pk,
+            "category": self.sid.category,
+            "name": "renamed",
+            "full_history_retention_period": (
+                self.sid.full_history_retention_period
+            ),
+            "metadata_only_retention_period": (
+                self.sid.metadata_only_retention_period
+            ),
+            "data": self.sid.data,
+            "links": self.get_links(self.sid),
+        }
+
+    @override_permission(Collection, "can_configure", AllowAll)
+    def test_patch_rename_singleton_ok(self) -> None:
         collection = self.scenario.workspace.get_singleton_collection(
             user=self.scenario.user, category=CollectionCategory.ARCHIVE
         )
-        response = self.post(
-            name=collection.name,
-            category=CollectionCategory(collection.category),
+        response = self.patch(
+            collection=collection,
             data={"name": collection.name},
         )
         new_data = self.assertAPIResponseOk(response)
         assert new_data == {
             "id": collection.pk,
+            "category": collection.category,
             "name": collection.name,
             "full_history_retention_period": (
                 collection.full_history_retention_period
@@ -971,13 +1222,16 @@ class CollectionViewTests(TestCase):
                 collection.metadata_only_retention_period
             ),
             "data": collection.data,
+            "links": self.get_links(collection),
         }
 
     @override_permission(Collection, "can_configure", AllowAll)
-    def test_post_rename_singleton_invalid_name(self) -> None:
-        response = self.post(
-            name="_",
-            category=CollectionCategory.ARCHIVE,
+    def test_patch_rename_singleton_invalid_name(self) -> None:
+        collection = self.scenario.workspace.get_singleton_collection(
+            user=self.scenario.user, category=CollectionCategory.ARCHIVE
+        )
+        response = self.patch(
+            collection=collection,
             data={"name": "renamed"},
         )
         self.assertResponseProblem(
@@ -988,3 +1242,92 @@ class CollectionViewTests(TestCase):
             ),
             status_code=status.HTTP_400_BAD_REQUEST,
         )
+        collection.refresh_from_db()
+        self.assertEqual(collection.name, "_")
+
+    def test_debusine_collection_iter(self) -> None:
+        debusine = self.get_debusine()
+        result = debusine.collection_iter(self.scenario.workspace.name)
+        self.assertCountEqual(
+            [(c.category, c.name) for c in result],
+            [
+                (CollectionCategory.PACKAGE_BUILD_LOGS, "_"),
+                (CollectionCategory.TASK_HISTORY, "_"),
+                (CollectionCategory.TASK_CONFIGURATION, "default"),
+                (CollectionCategory.ARCHIVE, "_"),
+                (CollectionCategory.SUITE, "sid"),
+                (CollectionCategory.SUITE, "trixie"),
+            ],
+        )
+
+    def test_debusine_collection_get(self) -> None:
+        debusine = self.get_debusine()
+        result = debusine.collection_get(
+            self.scenario.workspace.name, self.sid.category, self.sid.name
+        )
+        self.assertEqual(result.id, self.sid.id)
+        self.assertEqual(result.name, self.sid.name)
+        self.assertEqual(result.category, self.sid.category)
+        self.assertEqual(result.data, self.sid.data)
+
+    def test_debusine_collection_get_not_found(self) -> None:
+        debusine = self.get_debusine()
+        with self.assertRaisesRegex(DebusineError, r"Object not found"):
+            debusine.collection_get(
+                self.scenario.workspace.name,
+                self.sid.category,
+                "does-not-exist",
+            )
+
+    @override_permission(Workspace, "can_create_collections", AllowAll)
+    def test_debusine_create_collection(self) -> None:
+        debusine = self.get_debusine()
+        result = debusine.collection_create(
+            self.scenario.workspace.name,
+            CollectionDataNew(
+                name="new",
+                category=CollectionCategory.SUITE,
+                full_history_retention_period=12,
+                data={"created": True},
+            ),
+        )
+        new = Collection.objects.get(
+            workspace=self.scenario.workspace,
+            category=CollectionCategory.SUITE,
+            name="new",
+        )
+        self.assertEqual(result.id, new.pk)
+        self.assertEqual(result.name, "new")
+        self.assertEqual(result.category, CollectionCategory.SUITE)
+        self.assertEqual(result.full_history_retention_period, 12)
+        self.assertIsNone(result.metadata_only_retention_period)
+        self.assertEqual(result.data, {"created": True})
+
+        self.assertEqual(
+            new.full_history_retention_period, dt.timedelta(days=12)
+        )
+        self.assertIsNone(new.metadata_only_retention_period)
+        self.assertEqual(new.data, {"created": True})
+
+    @override_permission(Collection, "can_configure", AllowAll)
+    def test_debusine_collection_update(self) -> None:
+        debusine = self.get_debusine()
+        result = debusine.collection_update(
+            self.scenario.workspace.name,
+            CollectionData(
+                id=self.sid.id,
+                name=self.sid.name,
+                category=self.sid.category,
+                full_history_retention_period=3,
+                data=self.sid.data,
+            ),
+        )
+        self.assertEqual(result.id, self.sid.id)
+        self.assertEqual(result.name, self.sid.name)
+        self.assertEqual(result.category, self.sid.category)
+        self.assertEqual(result.full_history_retention_period, 3)
+        self.assertEqual(result.data, self.sid.data)
+        self.sid.refresh_from_db()
+        self.assertEqual(
+            self.sid.full_history_retention_period, dt.timedelta(days=3)
+        )
