@@ -9,12 +9,12 @@
 
 """Tests for the worker views."""
 
+import datetime as dt
 import logging
 import secrets
-from datetime import datetime
-from datetime import timezone as tz
 from typing import Any, ClassVar
 
+from django.db import transaction
 from django.db.models import Max
 from django.urls import reverse
 from django.utils import timezone
@@ -36,7 +36,7 @@ from debusine.tasks.models import (
     SbuildInput,
     WorkerType,
 )
-from debusine.test.django import TestCase
+from debusine.test.django import TestCase, TransactionTestCase
 
 
 class RegisterViewTests(TestCase):
@@ -206,7 +206,7 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
         )
 
     def create_work_request(
-        self, status: WorkRequest.Statuses, created_at: datetime
+        self, status: WorkRequest.Statuses, created_at: dt.datetime
     ) -> WorkRequest:
         """Return a new work_request as specified by the parameters."""
         environment_item = self.playground.create_debian_environment()
@@ -243,13 +243,13 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
         # Create WorkRequest pending
         work_request_pending = self.create_work_request(
             WorkRequest.Statuses.PENDING,
-            datetime(2022, 1, 5, 10, 13, 20, 204242, tz.utc),
+            dt.datetime(2022, 1, 5, 10, 13, 20, 204242, dt.timezone.utc),
         )
 
         # Create WorkRequest running
         work_request_running = self.create_work_request(
             WorkRequest.Statuses.RUNNING,
-            datetime(2022, 1, 5, 11, 14, 22, 242178, tz.utc),
+            dt.datetime(2022, 1, 5, 11, 14, 22, 242178, dt.timezone.utc),
         )
 
         # Request
@@ -276,13 +276,14 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
         """If a running work request cannot be retried, it is logged."""
         work_request_pending = self.create_work_request(
             WorkRequest.Statuses.PENDING,
-            datetime(2022, 1, 5, 10, 13, 20, 204242, tz.utc),
+            dt.datetime(2022, 1, 5, 10, 13, 20, 204242, dt.timezone.utc),
         )
         work_request_running = self.create_work_request(
             WorkRequest.Statuses.RUNNING,
-            datetime(2022, 1, 5, 11, 14, 22, 242178, tz.utc),
+            dt.datetime(2022, 1, 5, 11, 14, 22, 242178, dt.timezone.utc),
         )
         work_request_running.task_data = {"invalid": True}
+        work_request_running.configured_task_data = {"invalid": True}
         work_request_running.save()
 
         with self.assertLogsContains(
@@ -295,10 +296,10 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
                 reverse('api:work-request-get-next'),
                 headers={"token": self.token.key},
             )
+            self.assertAPIResponseOk(response)
 
         # The running work request was aborted but could not be retried; we
         # got the pending one.
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
         work_request_running.refresh_from_db()
         self.assertEqual(
             work_request_running.status, WorkRequest.Statuses.ABORTED
@@ -310,7 +311,7 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
         """Get a WorkRequest change the status to running."""
         work_request = self.create_work_request(
             WorkRequest.Statuses.PENDING,
-            datetime(2022, 1, 5, 10, 13, 20, 204242, tz.utc),
+            dt.datetime(2022, 1, 5, 10, 13, 20, 204242, dt.timezone.utc),
         )
 
         # Request
@@ -331,7 +332,7 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
         """Get a WorkRequest shows configured task data."""
         work_request = self.create_work_request(
             WorkRequest.Statuses.PENDING,
-            datetime(2022, 1, 5, 10, 13, 20, 204242, tz.utc),
+            dt.datetime(2022, 1, 5, 10, 13, 20, 204242, dt.timezone.utc),
         )
         work_request.task_data = {"test": 1}
         work_request.configured_task_data = {"test": 2}
@@ -353,12 +354,12 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
         # Create WorkRequest pending
         work_request_pending_01 = self.create_work_request(
             WorkRequest.Statuses.PENDING,
-            datetime(2022, 1, 5, 10, 13, 20, 204242, tz.utc),
+            dt.datetime(2022, 1, 5, 10, 13, 20, 204242, dt.timezone.utc),
         )
 
         work_request_pending_02 = self.create_work_request(
             WorkRequest.Statuses.PENDING,
-            datetime(2022, 1, 5, 10, 13, 22, 204242, tz.utc),
+            dt.datetime(2022, 1, 5, 10, 13, 22, 204242, dt.timezone.utc),
         )
         # Request
         response = self.client.get(
@@ -375,8 +376,8 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
 
         # Change created_at to force a change in the order that debusine
         # sends the WorkRequests to the client
-        work_request_pending_01.created_at = datetime(
-            2022, 1, 5, 10, 13, 24, 204242, tz.utc
+        work_request_pending_01.created_at = dt.datetime(
+            2022, 1, 5, 10, 13, 24, 204242, dt.timezone.utc
         )
 
         work_request_pending_01.save()
@@ -415,6 +416,72 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
         self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
 
 
+class GetNextWorkRequestViewTransactionTests(TransactionTestCase):
+    """Test transactional behaviour of GetNextWorkRequestView."""
+
+    def test_locks_workflow(self) -> None:
+        """The view locks all work requests in the affected workflows."""
+        with transaction.atomic():
+            workflows = [self.playground.create_workflow() for _ in range(5)]
+            work_requests = [
+                workflow.create_child_worker("noop")
+                for workflow in (
+                    workflows[0],
+                    workflows[0],
+                    workflows[1],
+                    workflows[1],
+                    workflows[2],
+                    workflows[3],
+                )
+            ]
+            # workflows[0] and workflows[1] have running work requests.
+            for work_request in (work_requests[0], work_requests[2]):
+                self.playground.advance_work_request(
+                    work_request, mark_running=True
+                )
+            assert work_requests[0].worker is not None
+            assert work_requests[0].worker.token is not None
+            # workflows[2] and workflows[3] have pending work requests
+            # assigned to the same worker as work_requests[0], with the one
+            # in workflows[2] having higher priority.
+            work_requests[4].priority_adjustment += 1
+            work_requests[4].save()
+            for work_request in (work_requests[4], work_requests[5]):
+                self.playground.advance_work_request(
+                    work_request, mark_pending=True
+                )
+                self.playground.assign_worker(
+                    work_request, worker=work_requests[0].worker
+                )
+
+        with transaction.atomic():
+            response = self.client.get(
+                reverse("api:work-request-get-next"),
+                headers={"token": work_requests[0].worker.token.key},
+            )
+
+            self.assertEqual(response.status_code, status.HTTP_200_OK)
+            # Each work request in each of the affected workflows is locked,
+            # but not unaffected work requests.
+            self.assert_work_requests_locked(
+                locked=[
+                    workflows[0],
+                    work_requests[0],
+                    work_requests[1],
+                    workflows[2],
+                    work_requests[4],
+                ],
+                unlocked=[
+                    workflows[1],
+                    work_requests[2],
+                    work_requests[3],
+                    workflows[3],
+                    work_requests[5],
+                    workflows[4],
+                ],
+            )
+
+
 class UpdateWorkRequestAsCompletedTests(TestCase):
     """Tests for UpdateWorkRequestAsCompleted class."""
 
@@ -578,6 +645,47 @@ class UpdateWorkRequestAsCompletedTests(TestCase):
         )
 
 
+class UpdateWorkRequestAsCompletedTransactionTests(TransactionTestCase):
+    """Test transactional behaviour of UpdateWorkRequestAsCompleted."""
+
+    def test_locks_workflow(self) -> None:
+        """The view locks all work requests in the workflow."""
+        with transaction.atomic():
+            workflow = self.playground.create_workflow()
+            work_requests = [
+                workflow.create_child_worker("noop") for _ in range(2)
+            ]
+            self.playground.advance_work_request(
+                work_requests[0], mark_running=True
+            )
+            assert work_requests[0].worker is not None
+            assert work_requests[0].worker.token is not None
+            unrelated = self.playground.create_worker_task()
+
+        with transaction.atomic():
+            response = self.client.put(
+                reverse(
+                    "api:work-request-completed",
+                    kwargs={"work_request_id": work_requests[0].id},
+                ),
+                data={"result": WorkRequest.Results.SUCCESS},
+                headers={"token": work_requests[0].worker.token.key},
+                content_type="application/json",
+            )
+
+            self.assertEqual(response.status_code, status.HTTP_200_OK)
+            work_requests[0].refresh_from_db()
+            self.assertEqual(
+                work_requests[0].status, WorkRequest.Statuses.COMPLETED
+            )
+            self.assertEqual(
+                work_requests[0].result, WorkRequest.Results.SUCCESS
+            )
+            self.assert_work_requests_locked(
+                locked=[workflow, *work_requests], unlocked=[unrelated]
+            )
+
+
 class UpdateWorkerDynamicMetadataTests(TestCase):
     """Tests for DynamicMetadata."""
 
