@@ -13,6 +13,11 @@ import itertools
 import logging
 from typing import Any
 
+try:
+    import pydantic.v1 as pydantic
+except ImportError:
+    import pydantic as pydantic  # type: ignore
+
 from django.http import Http404
 from django.http.response import HttpResponseBase
 from rest_framework import status
@@ -28,6 +33,7 @@ from rest_framework.response import Response
 from rest_framework.views import APIView
 
 from debusine.artifacts.models import TaskTypes
+from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
@@ -43,12 +49,15 @@ from debusine.db.models.work_requests import (
 )
 from debusine.server.exceptions import DebusineAPIException
 from debusine.server.serializers import (
+    WorkRequestConfirmSerializer,
     WorkRequestExternalDebsignRequestSerializer,
     WorkRequestExternalDebsignSerializer,
     WorkRequestSerializer,
+    WorkRequestSerializerWithConfiguredTaskData,
     WorkRequestUnblockSerializer,
     WorkRequestUpdateSerializer,
 )
+from debusine.server.tasks.wait import Confirm
 from debusine.server.tasks.wait.models import ExternalDebsignDynamicData
 from debusine.server.views.base import (
     BaseAPIView,
@@ -63,7 +72,7 @@ from debusine.server.views.rest import (
     IsTokenUserAuthenticated,
     IsWorkerAuthenticated,
 )
-from debusine.tasks import BaseTask
+from debusine.tasks import BaseTask, TaskConfigError
 from debusine.tasks.models import ActionTypes
 
 logger = logging.getLogger(__name__)
@@ -83,7 +92,7 @@ class WorkRequestView(
     permission_classes = [
         IsTokenUserAuthenticated | IsWorkerAuthenticated & IsGet
     ]
-    serializer_class = WorkRequestSerializer
+    serializer_class = WorkRequestSerializerWithConfiguredTaskData
     filter_backends = [CanDisplayFilterBackend]
     pagination_class = WorkRequestPagination
     lookup_url_kwarg = "work_request_id"
@@ -109,7 +118,7 @@ class WorkRequestView(
             return super().get(request)
 
         return Response(
-            WorkRequestSerializer(self.get_object()).data,
+            self.serializer_class(self.get_object()).data,
             status=status.HTTP_200_OK,
         )
 
@@ -118,7 +127,7 @@ class WorkRequestView(
         data = request.data.copy()
 
         data["created_by"] = request.auth.user.id
-        work_request_deserialized = WorkRequestSerializer(
+        work_request_deserialized = self.serializer_class(
             data=data,
             only_fields=[
                 'task_name',
@@ -145,19 +154,11 @@ class WorkRequestView(
         self.set_current_workspace(workspace)
         self.enforce(workspace.can_create_work_requests)
 
-        # only accept send-notification actions
-        for event, reactions in work_request_deserialized.validated_data.get(
-            "event_reactions_json", {}
-        ).items():
-            for action in reactions:
-                if action["action"] != ActionTypes.SEND_NOTIFICATION:
-                    raise DebusineAPIException(
-                        title="Invalid event_reactions",
-                        detail=(
-                            f"Action type {action['action']!r} is not allowed "
-                            f"here"
-                        ),
-                    )
+        self._validate_event_reactions(
+            work_request_deserialized.validated_data.get(
+                "event_reactions_json", {}
+            )
+        )
 
         if len(non_existing_channels := self._non_existing_channels(data)) > 0:
             raise DebusineAPIException(
@@ -178,7 +179,22 @@ class WorkRequestView(
         w = work_request_deserialized.save(task_type=task_type)
 
         try:
-            w.get_task().compute_dynamic_data(TaskDatabase(w))
+            task = w.get_task()
+        except TaskConfigError as exc:
+            exception: Exception = exc
+            errors = None
+            if isinstance(exc.original_exception, pydantic.ValidationError):
+                exception = exc.original_exception
+                errors = exc.original_exception.errors()
+
+            raise DebusineAPIException(
+                title='Cannot create task',
+                detail=f"Task data: {w.task_data} Error: {exception!r}",
+                pydantic_validation_errors=errors,
+            )
+
+        try:
+            task.compute_dynamic_data(TaskDatabase(w))
         except Exception as exc:
             if isinstance(exc, AssertionError):
                 logger.exception(
@@ -199,9 +215,24 @@ class WorkRequestView(
         w.refresh_from_db()
 
         return Response(
-            WorkRequestSerializer(w).data, status=status.HTTP_200_OK
+            self.serializer_class(w).data, status=status.HTTP_200_OK
         )
 
+    def _validate_event_reactions(
+        self, event_reactions: dict[str, Any]
+    ) -> None:
+        """Only accept send-notification actions."""
+        for event, reactions in event_reactions.items():
+            for action in reactions:
+                if action["action"] != ActionTypes.SEND_NOTIFICATION:
+                    raise DebusineAPIException(
+                        title="Invalid event_reactions",
+                        detail=(
+                            f"Action type {action['action']!r} is not allowed "
+                            f"here"
+                        ),
+                    )
+
     def _validate_task_name(
         self, task_name: str, is_superuser: bool
     ) -> TaskTypes:
@@ -483,6 +514,13 @@ class WorkRequestExternalDebsignView(
     ) -> HttpResponseBase:
         """Complete an `ExternalDebsign` task."""
         work_request = self.get_object()
+        WorkRequest.objects.filter(
+            id=work_request.id
+        ).lock_workflows_for_update()
+        user = context.require_user()
+        # This view's permission_classes ensure that the user can't be
+        # anonymous by this point.
+        assert user.is_authenticated
         serializer = WorkRequestExternalDebsignRequestSerializer(
             data=request.data
         )
@@ -542,6 +580,72 @@ class WorkRequestExternalDebsignView(
         )
         signed.created_by_work_request = work_request
         signed.save()
-        work_request.mark_completed(WorkRequest.Results.SUCCESS)
+        work_request.mark_completed(WorkRequest.Results.SUCCESS, user=user)
 
         return Response(self.get_serializer(work_request).data)
+
+
+class WorkRequestConfirmView(RetrieveAPIViewBase[WorkRequest], BaseAPIView):
+    """View used to handle ``Confirm`` wait tasks."""
+
+    permission_classes = [IsTokenUserAuthenticated]
+    serializer_class = WorkRequestConfirmSerializer
+    filter_backends = [CanDisplayFilterBackend]
+    lookup_url_kwarg = "work_request_id"
+
+    def get_queryset(self) -> WorkRequestQuerySet[Any]:
+        """Get the query set for this view."""
+        return WorkRequest.objects.in_current_scope()
+
+    def get_object(self) -> WorkRequest:
+        """Override to add constraints and return more API-friendly errors."""
+        try:
+            work_request = super().get_object()
+        except Http404 as exc:
+            raise NotFound(str(exc))
+
+        if (
+            work_request.task_type != TaskTypes.WAIT
+            or work_request.task_name != "confirm"
+        ):
+            raise ValidationError(
+                detail=(
+                    f"Expected work request to be Wait/Confirm; got "
+                    f"{work_request.task_type}/{work_request.task_name}"
+                )
+            )
+        if work_request.status != WorkRequest.Statuses.RUNNING:
+            raise ValidationError(
+                detail=(
+                    f"Work request is not running. "
+                    f"Status: {work_request.status}"
+                )
+            )
+
+        return work_request
+
+    def post(self, request: Request, *args: Any, **kwargs: Any) -> Response:
+        """Confirm or deny a work request."""
+        work_request = self.get_object()
+        self.set_current_workspace(work_request.workspace)
+        self.enforce(work_request.can_confirm)
+        task = work_request.get_task()
+        assert isinstance(task, Confirm)
+        user = context.require_user()
+        # Enforcing `can_confirm` ensures that the user can't be anonymous
+        # by this point.
+        assert user.is_authenticated
+        serializer = self.get_serializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+
+        task.respond(
+            user=user,
+            confirm=serializer.validated_data["confirm"],
+            comment=serializer.validated_data.get("comment"),
+        )
+        work_request.refresh_from_db()
+
+        return Response(
+            WorkRequestSerializerWithConfiguredTaskData(work_request).data,
+            status=status.HTTP_200_OK,
+        )
