@@ -65,6 +65,7 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
         Compute dynamic data for this workflow.
 
         :subject: package name of ``source_artifact``
+        :parameter_summary: name of debian suite: old_version → new_version
         """
         new_source_data = workflow_utils.source_package_data(self)
         new_source_package_name = get_source_package_name(new_source_data)
@@ -349,13 +350,11 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
         self,
     ) -> PackageArchLookupMap:
         """Return mapping of new binary packages by name and arch."""
-        workflow_root = self.work_request.get_workflow_root()
-
         new_binary_results = lookup_multiple(
             self.data.binary_artifacts,
             self.workspace,
             user=self.work_request.created_by,
-            workflow_root=workflow_root,
+            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
         packages_by_name_arch: defaultdict[
@@ -373,7 +372,8 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
                     for pkg_name, arch in self._package_arch_pairs(result):
                         packages_by_name_arch[pkg_name][arch] = (
                             reconstruct_lookup(
-                                result, workflow_root=workflow_root
+                                result,
+                                workflow_root=self.work_request.workflow_root,
                             )
                         )
 
