@@ -702,10 +702,12 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
                         [
                             f"{bookworm_lookup}/name:hello_1.0.0_amd64",
                         ],
-                        [
-                            self.new_binary_artifact_lookup("amd64"),
-                            self.new_binary_artifact_lookup("all"),
-                        ],
+                        sorted(
+                            [
+                                self.new_binary_artifact_lookup("amd64"),
+                                self.new_binary_artifact_lookup("all"),
+                            ]
+                        ),
                     ]
                 },
             },
