@@ -150,23 +150,25 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
     def test_create_orchestrator(self) -> None:
         """A PackageUploadWorkflow can be instantiated."""
-        source_artifact = self.create_source_package().id
-        binary_artifacts = ["internal@collections/name:build-arm64"]
+        source_artifact, [binary_artifact] = (
+            self.create_source_binary_artifacts(["arm64"])
+        )
         target_distribution = "debian:bookworm"
 
         w = self.create_package_upload_workflow(
             extra_task_data={
-                "source_artifact": source_artifact,
-                "binary_artifacts": binary_artifacts,
+                "source_artifact": source_artifact.id,
+                "binary_artifacts": [binary_artifact.id],
                 "target_distribution": target_distribution,
                 "vendor": "debian",
                 "codename": "bookworm",
             }
         )
 
-        self.assertEqual(w.data.source_artifact, source_artifact)
+        self.assertEqual(w.data.source_artifact, source_artifact.id)
         self.assertEqual(
-            w.data.binary_artifacts, LookupMultiple.parse_obj(binary_artifacts)
+            w.data.binary_artifacts,
+            LookupMultiple.parse_obj([binary_artifact.id]),
         )
 
         self.assertEqual(w.data.target_distribution, target_distribution)
@@ -183,7 +185,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
             template,
             task_data={**{"target": "sftp://upload.example.org/queue"}, **data},
         )
-        orchestrate_workflow(wr)
+        self.assertTrue(orchestrate_workflow(wr))
 
         return wr
 
@@ -388,7 +390,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
             },
             parent=parent,
         )
-        orchestrate_workflow(wr)
+        self.assertTrue(orchestrate_workflow(wr))
 
         self.assertEqual(
             wr.children.filter(task_name="mergeuploads").count(), 1
@@ -459,7 +461,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
             name=f"build-{architectures[0]}",
             replace=True,
         )
-        orchestrate_workflow(wr)
+        self.assertTrue(orchestrate_workflow(wr))
 
         self.assertEqual(
             wr.children.filter(task_name="mergeuploads").count(), 1
