@@ -433,8 +433,7 @@ class WorkflowUtilsTests(TestCase):
         )
 
         with self.assertRaisesRegex(
-            workflow_utils.ArtifactHasNoArchitecture,
-            re.escape(f"{DebianSourcePackage!r}"),
+            ValueError, f"Unexpected type: {DebianSourcePackage.__name__}"
         ):
             workflow_utils.lookup_result_architecture(result)
 
