@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -22,11 +22,15 @@ from debusine.signing.tasks.debsign import Debsign  # noqa: I202
 from debusine.signing.tasks.generate_key import GenerateKey  # noqa: I202
 from debusine.signing.tasks.noop import SigningNoop  # noqa: I202
 from debusine.signing.tasks.sign import Sign  # noqa: I202
+from debusine.signing.tasks.sign_repository_index import (  # noqa: I202
+    SignRepositoryIndex,
+)
 
 __all__ = [
     "BaseSigningTask",
     "Debsign",
     "GenerateKey",
-    "SigningNoop",
     "Sign",
+    "SignRepositoryIndex",
+    "SigningNoop",
 ]
