@@ -21,6 +21,7 @@ name of the class must be unique among all child classes.
 A child class must, at the very least, override the :py:meth:`BaseTask.execute`
 method.
 """
+import datetime as dt
 import inspect
 import logging
 import os
@@ -33,7 +34,6 @@ from abc import ABCMeta, abstractmethod
 from collections import defaultdict
 from collections.abc import Collection, Generator
 from contextlib import contextmanager
-from datetime import datetime, timezone
 from pathlib import Path
 from typing import (
     Any,
@@ -55,7 +55,6 @@ if TYPE_CHECKING:
     from _typeshed import OpenBinaryModeWriting, OpenTextModeWriting
     from debusine.db.models import WorkRequest
 
-
 from debian.debian_support import Version
 
 from debusine.artifacts import WorkRequestDebugLogs
@@ -230,14 +229,14 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         dynamic_task_data: dict[str, Any] | None = None,
     ) -> None:
         """Initialize the task."""
-        #: Validated task data submitted through :py:meth:`configure` without
-        # BaseTask generic data
-        self._configure(task_data, dynamic_task_data)
-
         #: A :py:class:`logging.Logger` instance that can be used in child
         #: classes when you override methods to implement the task.
         self.logger = logging.getLogger("debusine.tasks")
 
+        #: Validated task data submitted through :py:meth:`configure` without
+        # BaseTask generic data
+        self._configure(task_data, dynamic_task_data)
+
         # Task is aborted: the task does not need to be executed, and can be
         # stopped if it is already running
         self._aborted = False
@@ -859,7 +858,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
                     "Work request %s: %s", self.work_request_id, message
                 )
                 print(
-                    f"{datetime.now(timezone.utc).isoformat()} {message}",
+                    f"{dt.datetime.now(dt.timezone.utc).isoformat()} {message}",
                     file=stages_log,
                 )
 
@@ -964,7 +963,7 @@ def get_environment(
     lookup: LookupSingle,
     *,
     architecture: str | None = None,
-    backend: str | None = None,
+    backend: BackendType | None = None,
     default_category: CollectionCategory | None = None,
     image_category: ExecutorImageCategory | None = None,
     try_variant: str | None = None,
@@ -1150,10 +1149,10 @@ class BaseTaskWithExecutor(BaseExternalTask[TDE, DTDE], Generic[TDE, DTDE]):
         self.executor_instance.start()
 
     @property
-    def backend(self) -> str:
+    def backend(self) -> BackendType:
         """Return the backend name to use."""
         backend = self.data.backend
-        if backend == "auto":
+        if backend == BackendType.AUTO:
             backend = self.DEFAULT_BACKEND
         return backend
 
@@ -1577,6 +1576,8 @@ class ExtraRepositoryMixin(
 
     # Deb822 style sources:
     extra_repository_sources: list[Path]
+    # One-line sources:
+    extra_repository_lists: list[Path]
     # GPG keys (for apt < 2.3.10):
     extra_repository_keys: list[Path]
 
@@ -1584,6 +1585,7 @@ class ExtraRepositoryMixin(
         """Initialize class variables."""
         super().__init__(*args, **kwargs)
         self.extra_repository_sources = []
+        self.extra_repository_lists = []
         self.extra_repository_keys = []
 
     def supports_deb822_sources(self, codename: str) -> bool:
@@ -1608,32 +1610,39 @@ class ExtraRepositoryMixin(
         self, codename: str, destination: Path
     ) -> None:
         """
-        Write extra_repositories config to files in destination.
+        Write ``extra_repositories`` config to files in destination.
 
-        extra_repository_keys will be populated with keys to install into
-        /etc/apt/keyrings, if needed.
+        ``extra_repository_keys`` will be populated with keys to install
+        into /etc/apt/keyrings, if needed.
 
-        extra_repository_sources will be populated with deb822 sources files,
-        if supported.
+        If deb822 sources files are supported, ``extra_repository_sources``
+        will be populated with them; otherwise, ``extra_repository_lists``
+        will be populated with one-line sources.
         """
         inline_signed_by = self.supports_inline_signed_by(codename)
         for i, extra_repo in enumerate(self.data.extra_repositories or []):
+            signed_by = None
             if not inline_signed_by:
                 if extra_repo.signing_key:
                     path = destination / f"extra_apt_key_{i}.asc"
                     path.write_text(extra_repo.signing_key + "\n")
                     self.extra_repository_keys.append(path)
+                    signed_by = f"/etc/apt/keyrings/{path.name}"
 
             # apt < 1.1 has no support for deb822 sources
             if self.supports_deb822_sources(codename):
                 path = destination / f"extra_repository_{i}.sources"
-                signed_by = None
-                if not inline_signed_by:
-                    signed_by = f"/etc/apt/keyrings/extra_apt_key_{i}.asc"
                 path.write_text(
                     extra_repo.as_deb822_source(signed_by_filename=signed_by)
                 )
                 self.extra_repository_sources.append(path)
+            else:
+                path = destination / f"extra_repository_{i}.list"
+                path.write_text(
+                    extra_repo.as_oneline_source(signed_by_filename=signed_by)
+                    + "\n"
+                )
+                self.extra_repository_lists.append(path)
 
     def iter_oneline_sources(self) -> Generator[str]:
         """
