@@ -30,6 +30,7 @@ from debusine.tasks.executors.base import (
     executor_class,
 )
 from debusine.tasks.executors.unshare import UnshareExecutor, UnshareInstance
+from debusine.tasks.models import BackendType
 from debusine.tasks.tests.helper_mixin import ExternalTaskHelperMixin
 from debusine.test import TestCase
 
@@ -39,6 +40,7 @@ class UnshareExecutorTests(ExternalTaskHelperMixin[Noop], TestCase):
 
     def setUp(self) -> None:
         """Mock the Debusine API for tests."""
+        super().setUp()
         self.debusine_api = MagicMock(spec=Debusine)
         self.image_artifact = self.mock_image_download(self.debusine_api)
 
@@ -46,16 +48,21 @@ class UnshareExecutorTests(ExternalTaskHelperMixin[Noop], TestCase):
 
     def test_backend_name(self) -> None:
         """Test that the backend_name attribute was set."""
-        self.assertEqual(UnshareExecutor.backend_name, "unshare")
+        self.assertEqual(UnshareExecutor.backend_name, BackendType.UNSHARE)
 
     def test_available(self) -> None:
         """Test that available() returns True if unshare is available."""
-        self.mock_is_command_available({"unshare": True})
+        self.mock_is_command_available({"unshare": True, "ip": True})
         self.assertTrue(UnshareExecutor.available())
 
     def test_available_no_unshare(self) -> None:
         """Test that available() returns False if unshare is not available."""
-        self.mock_is_command_available({"unshare": False})
+        self.mock_is_command_available({"unshare": False, "ip": True})
+        self.assertFalse(UnshareExecutor.available())
+
+    def test_available_no_iproute2(self) -> None:
+        """Test that available() returns False if unshare is not available."""
+        self.mock_is_command_available({"unshare": True, "ip": False})
         self.assertFalse(UnshareExecutor.available())
 
     def test_instantiation_fetches_artifact(self) -> None:
@@ -107,7 +114,7 @@ class UnshareExecutorTests(ExternalTaskHelperMixin[Noop], TestCase):
 
     def test_executor_class_finds_unshare(self) -> None:
         """Test that executor_class() supports unshare."""
-        instance = executor_class("unshare")
+        instance = executor_class(BackendType.UNSHARE)
         self.assertEqual(instance, UnshareExecutor)
 
     def test_create(self) -> None:
@@ -175,6 +182,7 @@ class UnshareInstanceTests(TestCase):
 
     def setUp(self) -> None:
         """Initialize test."""
+        super().setUp()
         self.image_file = self.create_temporary_file()
 
         self.instance = UnshareInstance(self.image_file)
@@ -188,6 +196,7 @@ class UnshareInstanceTests(TestCase):
         """Remove any unpacked instances during tests."""
         if self.instance._extracted:
             rmtree(self.instance._extracted)
+        super().tearDown()
 
     def patch_run_unshare_command(self) -> tuple[Any, MagicMock]:
         """Patch _run_share_cmd() and return its patcher and mock."""
@@ -430,6 +439,52 @@ class UnshareInstanceTests(TestCase):
         self.assertEqual(stderr_file_path.read_text(), stderr_contents)
         self.assertEqual(stdout_file_path.read_text(), stdout_contents)
 
+    def test_run_unshare_command_environment(self) -> None:
+        """run_unshare_command() runs a command in a sanitized environment."""
+        self.run_unshare_command_patcher.stop()
+        # Yes, this runs "env env", but "sh -c env" would (annoyingly) set PWD
+        self.patch_unshare_command_args("env", [])
+
+        completed = UnshareInstance._run_unshare_command(["env"], text=True)
+        output = (
+            "PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
+            "\n"
+        )
+        self.assertEqual(completed.stdout, output)
+        self.assertEqual(completed.returncode, 0)
+
+    def test_run_unshare_command_env_none(self) -> None:
+        """env=None is the same as not specifying env."""
+        self.run_unshare_command_patcher.stop()
+        # Yes, this runs "env env", but "sh -c env" would (annoyingly) set PWD
+        self.patch_unshare_command_args("env", [])
+
+        completed = UnshareInstance._run_unshare_command(
+            ["env"], text=True, env=None
+        )
+        output = (
+            "PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
+            "\n"
+        )
+        self.assertIn(completed.stdout, output)
+        self.assertEqual(completed.returncode, 0)
+
+    def test_run_unshare_command_environment_override(self) -> None:
+        """run_unshare_command() lets the caller specify an environment."""
+        self.run_unshare_command_patcher.stop()
+        self.patch_unshare_command_args("env", [])
+
+        completed = UnshareInstance._run_unshare_command(
+            ["env"], text=True, env={"FOO": "BAR"}
+        )
+        output = (
+            "FOO=BAR\n"
+            "PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
+            "\n"
+        )
+        self.assertEqual(completed.stdout, output)
+        self.assertEqual(completed.returncode, 0)
+
     def test_root(self) -> None:
         """_root property returns self._extracted / root."""
         self.instance._extracted = self.create_temporary_directory()
