@@ -12,6 +12,7 @@
 import hashlib
 from collections.abc import Iterable
 from pathlib import Path
+from typing import Any
 from unittest import mock
 from unittest.mock import call
 
@@ -54,6 +55,47 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
             self.task._debug_log_files_directory.cleanup()
         super().tearDown()
 
+    def configure_task(
+        self,
+        task_data: dict[str, Any] | None = None,
+        override: dict[str, Any] | None = None,
+        remove: list[str] | None = None,
+    ) -> None:
+        """Perform further setup."""
+        super().configure_task(task_data, override, remove)
+
+        self.task_db = FakeTaskDatabase(
+            multiple_lookups={
+                # input.uploads
+                (LookupMultiple.parse_obj([421, 666]), None): [
+                    ArtifactInfo(
+                        id=421,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Files": [{"name": "hello_1.0_amd64.deb"}],
+                                "Source": "hello",
+                            },
+                        ),
+                    ),
+                    ArtifactInfo(
+                        id=666,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "s390x",
+                                "Files": [{"name": "hello_1.0_s390x.deb"}],
+                                "Source": "hello",
+                            },
+                        ),
+                    ),
+                ]
+            },
+        )
+
     @staticmethod
     def _make_files_field(paths: Iterable[Path]) -> list[dict[str, str]]:
         """Make a valid ``Files`` field."""
@@ -86,111 +128,50 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
 
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
-        binary_artifacts_lookup = LookupMultiple.parse_obj([421, 666])
-        task_db = FakeTaskDatabase(
-            multiple_lookups={
-                # input.uploads
-                (binary_artifacts_lookup, None): [
-                    ArtifactInfo(
-                        id=421,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "amd64",
-                                "Files": [{"name": "hello_1.0_amd64.deb"}],
-                                "Source": "hello",
-                            },
-                        ),
-                    ),
-                    ArtifactInfo(
-                        id=666,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "s390x",
-                                "Files": [{"name": "hello_1.0_s390x.deb"}],
-                                "Source": "hello",
-                            },
-                        ),
-                    ),
-                ]
-            },
-        )
-
         self.assertEqual(
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
             MergeUploadsDynamicData(
-                input_uploads_ids=[421, 666], subject="hello"
+                input_uploads_ids=[421, 666],
+                subject="hello",
+                parameter_summary="hello (amd64, s390x)",
             ),
         )
 
-    def test_compute_dynamic_data_subject_is_none(self) -> None:
-        """Uploading packages with different source package names."""
+    def test_compute_dynamic_data_multiple_packages(self) -> None:
+        """Dynamic data receives relevant artifact IDs."""
         binary_artifacts_lookup = LookupMultiple.parse_obj([421, 666])
-        task_db = FakeTaskDatabase(
-            multiple_lookups={
-                # input.uploads
-                (binary_artifacts_lookup, None): [
-                    ArtifactInfo(
-                        id=421,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "amd64",
-                                "Files": [{"name": "hello_1.0_amd64.deb"}],
-                                "Source": "hello",
-                            },
-                        ),
-                    ),
-                    ArtifactInfo(
-                        id=666,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "s390x",
-                                "Files": [{"name": "hello_1.0_s390x.deb"}],
-                                "Source": "python",
-                            },
-                        ),
-                    ),
-                ]
-            },
+        self.task_db.multiple_lookups[(binary_artifacts_lookup, None)][1] = (
+            ArtifactInfo(
+                id=666,
+                category=ArtifactCategory.UPLOAD,
+                data=DebianUpload(
+                    type="dpkg",
+                    changes_fields={
+                        "Architecture": "s390x",
+                        "Files": [{"name": "bye_1.0-1_s390x.deb"}],
+                        "Source": "bye",
+                    },
+                ),
+            )
         )
 
         self.assertEqual(
-            self.task.compute_dynamic_data(task_db),
-            MergeUploadsDynamicData(input_uploads_ids=[421, 666], subject=None),
+            self.task.compute_dynamic_data(self.task_db),
+            MergeUploadsDynamicData(
+                input_uploads_ids=[421, 666],
+                subject=None,
+                parameter_summary=None,
+            ),
         )
 
     def test_compute_dynamic_raise_task_error_wrong_category(self) -> None:
         binary_artifacts_lookup = LookupMultiple.parse_obj([421, 666])
-        task_db = FakeTaskDatabase(
-            multiple_lookups={
-                # input.uploads
-                (binary_artifacts_lookup, None): [
-                    ArtifactInfo(
-                        id=421,
-                        category=ArtifactCategory.SYSTEM_TARBALL,
-                        data=EmptyArtifactData(),
-                    ),
-                    ArtifactInfo(
-                        id=666,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "s390x",
-                                "Files": [{"name": "hello_1.0_s390x.deb"}],
-                                "Source": "python",
-                            },
-                        ),
-                    ),
-                ]
-            },
+        self.task_db.multiple_lookups[(binary_artifacts_lookup, None)][0] = (
+            ArtifactInfo(
+                id=421,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=EmptyArtifactData(),
+            )
         )
 
         with self.assertRaisesRegex(
@@ -198,7 +179,7 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
             r"^input.uploads: unexpected artifact category: "
             r"'debian:system-tarball'. Valid categories: \['debian:upload'\]$",
         ):
-            self.task.compute_dynamic_data(task_db)
+            self.task.compute_dynamic_data(self.task_db)
 
     def test_get_input_artifacts_ids(self) -> None:
         """Test get_input_artifacts_ids."""
@@ -764,7 +745,3 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
                 call(uploaded_artifacts[0].id, 2, "extends"),
             ]
         )
-
-    def test_get_label(self) -> None:
-        """Test get_label."""
-        self.assertEqual(self.task.get_label(), "merge package uploads")
