@@ -9,9 +9,9 @@
 
 """Unit tests for task models."""
 
+import datetime as dt
 import re
 import textwrap
-from datetime import datetime, timezone
 from typing import get_args
 from unittest import TestCase
 
@@ -836,14 +836,14 @@ class ImageCacheUsageLogEntryTests(TestCase):
     def test_timestamp_validation_succeeds(self) -> None:
         """Test ImageCacheUsageLogEntry accepts aware timestamp."""
         task_models.ImageCacheUsageLogEntry(
-            filename="foo", timestamp=datetime.now(timezone.utc)
+            filename="foo", timestamp=dt.datetime.now(dt.timezone.utc)
         )
 
     def test_timestamp_validation_fails(self) -> None:
         """Test ImageCacheUsageLogEntry rejects naive timestamp."""
         with self.assertRaisesRegex(ValueError, "timestamp is TZ-naive"):
             task_models.ImageCacheUsageLogEntry(
-                filename="foo", timestamp=datetime.now()
+                filename="foo", timestamp=dt.datetime.now()
             )
 
 
