@@ -10,10 +10,10 @@
 """Tests for debusine.tests.django."""
 
 import contextlib
+import datetime as dt
 import json
 import re
 from collections.abc import Callable, Generator
-from datetime import timedelta
 from pathlib import Path
 from typing import Any, ClassVar, Never, Self
 from unittest import mock
@@ -229,7 +229,7 @@ class TestCaseTests(TestCase):
         """create_artifact() set expire_at use correct expire_at.."""
         now = timezone.now()
         artifact, _ = self.playground.create_artifact(expiration_delay=1)
-        artifact.created_at = now - timedelta(days=1)
+        artifact.created_at = now - dt.timedelta(days=1)
         artifact.save()
         artifact.refresh_from_db()
         self.assertEqual(artifact.expire_at, now)
