@@ -9,7 +9,7 @@
 
 """Custom path converters for package archives."""
 
-from datetime import datetime, timezone
+import datetime as dt
 
 from django.urls import register_converter
 
@@ -20,13 +20,13 @@ class SnapshotConverter:
     regex = r"[0-9]{8}T[0-9]{6}Z"
     _timestamp_format = "%Y%m%dT%H%M%SZ"
 
-    def to_python(self, value: str) -> datetime:
+    def to_python(self, value: str) -> dt.datetime:
         """Convert the matched string to a timestamp."""
-        return datetime.strptime(value, self._timestamp_format).replace(
-            tzinfo=timezone.utc
+        return dt.datetime.strptime(value, self._timestamp_format).replace(
+            tzinfo=dt.timezone.utc
         )
 
-    def to_url(self, value: datetime) -> str:
+    def to_url(self, value: dt.datetime) -> str:
         """Convert a timestamp into a string to be used in a URL."""
         return value.strftime(self._timestamp_format)
 
