@@ -46,6 +46,7 @@ from debusine.web.views.files import (
 )
 from debusine.web.views.http_errors import HttpError400, catch_http_errors
 from debusine.web.views.mixins import UIShortcutsMixin
+from debusine.web.views.places import Place
 from debusine.web.views.view_utils import format_yaml
 
 
@@ -87,9 +88,9 @@ class ArtifactDetailView(ArtifactView, DetailViewBase[Artifact]):
         """Add the select_related we need to the queryset."""
         return super().get_queryset().select_related("workspace__scope")
 
-    def get_title(self) -> str:
-        """Return the page title."""
-        return f"Artifact {self.object.get_label()}"
+    def get_place(self) -> Place:
+        """Return the Place for this page."""
+        return self.object.ui(self.request).place
 
     def get_main_ui_shortcuts(self) -> list[ui_shortcuts.UIShortcut]:
         """Return a list of UI shortcuts for this view."""
@@ -200,6 +201,7 @@ class FileView(
     FileDownloadMixin,
     ArtifactView,
     SingleObjectMixinBase[FileInArtifact],
+    abc.ABC,
 ):
     """Base for all FileInArtifact views."""
 
@@ -231,13 +233,6 @@ class FileView(
         """Artifact for this view."""
         return self.get_object().artifact
 
-    def get_title(self) -> str:
-        """Return the page title."""
-        title = self.object.path
-        if not self.object.complete:
-            title += " (incomplete)"
-        return title
-
     def get_main_ui_shortcuts(self) -> list[ui_shortcuts.UIShortcut]:
         """Return a list of UI shortcuts for this view."""
         shortcuts = super().get_main_ui_shortcuts()
@@ -261,10 +256,18 @@ class FileDetailView(FileView, DetailViewBase[FileInArtifact]):
     template_name = "web/fileinartifact-detail.html"
     object: FileInArtifact
 
+    def get_place(self) -> Place:
+        """Return the Place for this page."""
+        return self.object.ui(self.request).place
+
 
 class FileDetailViewRaw(FileView):
     """Download a file from an artifact, displayed inline."""
 
+    def get_place(self) -> Place:
+        """Return the Place for this page."""
+        return self.object.ui(self.request).place_raw
+
     def get(
         self, request: HttpRequest, **kwargs: Any  # noqa: U100
     ) -> HttpResponseBase:
