@@ -14,6 +14,7 @@ from typing import Any
 
 from django.db.models import Count, Q, QuerySet
 from django.http import Http404
+from django.urls import reverse
 
 from debusine.db.context import context
 from debusine.db.models import Collection, CollectionItem
@@ -26,6 +27,7 @@ from debusine.web.views.base import (
     WorkspaceView,
 )
 from debusine.web.views.mixins import RightbarUIMixin, UIShortcutsMixin
+from debusine.web.views.places import Place
 from debusine.web.views.table import TableMixin
 from debusine.web.views.tables import CollectionItemTable
 from debusine.web.views.view_utils import format_yaml
@@ -39,9 +41,11 @@ class CollectionListView(WorkspaceView, ListViewBase[Collection]):
     context_object_name = "collection_list"
     ordering = ["category", "name"]
 
-    def get_title(self) -> str:
-        """Return the page title."""
-        return f"{context.require_workspace().name} collections"
+    def get_place(self) -> Place:
+        """Return the Place for this page."""
+        return (
+            context.require_workspace().ui(self.request).place_collection_list
+        )
 
     def get_queryset(self) -> QuerySet[Collection]:
         """Filter collection by accessible workspace."""
@@ -59,11 +63,15 @@ class CollectionCategoryListView(WorkspaceView, ListViewBase[Collection]):
     context_object_name = "collection_list"
     ordering = ["name"]
 
-    def get_title(self) -> str:
-        """Return the page title."""
-        return (
-            f"{context.require_workspace().name}"
-            f" {self.kwargs['ccat']} collections"
+    def get_place(self) -> Place:
+        """Return the Place for this page."""
+        workspace = context.require_workspace()
+        return Place(
+            title=f"{workspace.name} {self.kwargs['ccat']} collections",
+            url=reverse(
+                "workspaces:collections:category_list",
+                kwargs={"wname": workspace.name, "ccat": self.kwargs["ccat"]},
+            ),
         )
 
     def get_queryset(self) -> QuerySet[Collection]:
@@ -122,6 +130,10 @@ class CollectionDetailView(CollectionViewMixin, DetailViewBase[Collection]):
     template_name = "web/collection-detail.html"
     context_object_name = "collection"
 
+    def get_place(self) -> Place:
+        """Return the Place for this page."""
+        return self.object.ui(self.request).place
+
     def get_object(
         self, queryset: QuerySet[Collection] | None = None  # noqa: U100
     ) -> Collection:
@@ -188,6 +200,10 @@ class CollectionSearchView(
     table_class = CollectionItemTable
     paginate_by = 50
 
+    def get_place(self) -> Place:
+        """Return the Place for this page."""
+        return self.collection.ui(self.request).place_search
+
     def get_queryset(self) -> QuerySet[CollectionItem]:
         """Filter items based on the search form."""
         queryset = (
@@ -253,9 +269,9 @@ class CollectionItemDetailView(
     template_name = "web/collection-item-detail.html"
     context_object_name = "item"
 
-    def get_title(self) -> str:
-        """Return page title."""
-        return self.object.name
+    def get_place(self) -> Place:
+        """Return the Place for this page."""
+        return self.object.ui(self.request).place
 
     def get_sidebar_items(self) -> list[sidebar.SidebarItem]:
         """Return a list of sidebar items."""
