@@ -19,15 +19,16 @@ from debusine.tasks.models import (
     AutopkgtestData,
     AutopkgtestDynamicData,
     AutopkgtestInput,
+    BackendType,
     LookupMultiple,
     LookupSingle,
 )
 from debusine.test.django import TestCase
-from debusine.web.views.autopkgtest import AutopkgtestView
+from debusine.web.views.autopkgtest import AutopkgtestViewWorkRequestPlugin
 
 
-class AutopkgtestViewTests(TestCase):
-    """Tests for AutopkgtestView."""
+class AutopkgtestViewWorkRequestPluginTests(TestCase):
+    """Tests for AutopkgtestViewWorkRequestPlugin."""
 
     def create_work_request_for_artifacts(
         self,
@@ -50,7 +51,7 @@ class AutopkgtestViewTests(TestCase):
             environments_collection.manager.add_artifact(
                 environment_artifact,
                 user=self.playground.get_default_user(),
-                variables={"backend": "unshare"},
+                variables={"backend": BackendType.UNSHARE},
             )
             environment_lookup = "{vendor}/match:codename={codename}".format(
                 **environment_artifact.data
@@ -146,14 +147,14 @@ class AutopkgtestViewTests(TestCase):
         ).dict()
         work_request.save()
         result_artifact = self.create_autopkgtest_artifact(work_request)
-        autopkgtest_view = AutopkgtestView(work_request)
+        autopkgtest_view = AutopkgtestViewWorkRequestPlugin(work_request)
         context_data = autopkgtest_view.get_context_data()
 
         self.assertEqual(
             context_data,
             {
                 "request_data": {
-                    "backend": "auto",
+                    "backend": BackendType.AUTO,
                     "binary_artifacts": {
                         "lookup": [binary_artifact_1.id, binary_artifact_2.id],
                         "artifacts": [
@@ -247,7 +248,7 @@ class AutopkgtestViewTests(TestCase):
             ],
         ).dict()
         work_request.save()
-        autopkgtest_view = AutopkgtestView(work_request)
+        autopkgtest_view = AutopkgtestViewWorkRequestPlugin(work_request)
         context_data = autopkgtest_view.get_context_data()
 
         self.assertEqual(
@@ -289,7 +290,7 @@ class AutopkgtestViewTests(TestCase):
         work_request: WorkRequest = self.create_work_request_for_artifacts(
             environment_artifact, source_artifact, []
         )
-        autopkgtest_view = AutopkgtestView(work_request)
+        autopkgtest_view = AutopkgtestViewWorkRequestPlugin(work_request)
         context_data = autopkgtest_view.get_context_data()
         self.assertCountEqual(
             context_data.keys(), {"request_data", "result", "specialized_tab"}
@@ -309,7 +310,7 @@ class AutopkgtestViewTests(TestCase):
         work_request: WorkRequest = self.create_work_request_for_artifacts(
             environment_artifact, source_artifact, []
         )
-        autopkgtest_view = AutopkgtestView(work_request)
+        autopkgtest_view = AutopkgtestViewWorkRequestPlugin(work_request)
         context_data = autopkgtest_view.get_context_data()
         self.assertIsNone(context_data["request_data"]["codename"])
         self.assertIsNone(context_data["request_data"]["vendor"])
@@ -348,7 +349,7 @@ class AutopkgtestViewTests(TestCase):
         ).dict()
         work_request.save()
         environment_artifact.delete()
-        autopkgtest_view = AutopkgtestView(work_request)
+        autopkgtest_view = AutopkgtestViewWorkRequestPlugin(work_request)
         context_data = autopkgtest_view.get_context_data()
         self.assertIsNone(context_data["request_data"]["codename"])
         self.assertIsNone(context_data["request_data"]["vendor"])
@@ -378,7 +379,7 @@ class AutopkgtestViewTests(TestCase):
             input_binary_artifacts_ids=[],
         ).dict()
         work_request.save()
-        autopkgtest_view = AutopkgtestView(work_request)
+        autopkgtest_view = AutopkgtestViewWorkRequestPlugin(work_request)
         context_data = autopkgtest_view.get_context_data()
         self.assertIsNone(context_data["request_data"]["codename"])
         self.assertIsNone(context_data["request_data"]["vendor"])
@@ -409,7 +410,7 @@ class AutopkgtestViewTests(TestCase):
         source_artifact_id = source_artifact.id
         source_artifact.files.clear()
         source_artifact.delete()
-        autopkgtest_view = AutopkgtestView(work_request)
+        autopkgtest_view = AutopkgtestViewWorkRequestPlugin(work_request)
         context_data = autopkgtest_view.get_context_data()
         self.assertEqual(
             context_data["request_data"]["source_artifact"],
@@ -448,7 +449,7 @@ class AutopkgtestViewTests(TestCase):
             input_binary_artifacts_ids=[],
         ).dict()
         work_request.save()
-        autopkgtest_view = AutopkgtestView(work_request)
+        autopkgtest_view = AutopkgtestViewWorkRequestPlugin(work_request)
         context_data = autopkgtest_view.get_context_data()
         self.assertIsNone(context_data["request_data"]["source_name"])
         self.assertIsNone(context_data["request_data"]["source_version"])
@@ -489,7 +490,7 @@ class AutopkgtestViewTests(TestCase):
         binary_artifact_1_id = binary_artifact_1.id
         binary_artifact_1.files.clear()
         binary_artifact_1.delete()
-        autopkgtest_view = AutopkgtestView(work_request)
+        autopkgtest_view = AutopkgtestViewWorkRequestPlugin(work_request)
         context_data = autopkgtest_view.get_context_data()
         self.assertEqual(
             context_data["request_data"]["binary_artifacts"],
@@ -532,7 +533,7 @@ class AutopkgtestViewTests(TestCase):
         ).dict()
         work_request.save()
 
-        autopkgtest_view = AutopkgtestView(work_request)
+        autopkgtest_view = AutopkgtestViewWorkRequestPlugin(work_request)
         context_data = autopkgtest_view.get_context_data()
 
         self.assertEqual(
@@ -673,7 +674,9 @@ class AutopkgtestViewTests(TestCase):
         for fail_on, expected_scenarios in sub_tests:
             with self.subTest(fail_on=fail_on):
                 self.assertEqual(
-                    AutopkgtestView._get_fail_on_scenarios(fail_on),
+                    AutopkgtestViewWorkRequestPlugin._get_fail_on_scenarios(
+                        fail_on
+                    ),
                     expected_scenarios,
                 )
 
@@ -701,7 +704,7 @@ class AutopkgtestViewTests(TestCase):
         )
 
         # Try unconfigured
-        autopkgtest_view = AutopkgtestView(work_request)
+        autopkgtest_view = AutopkgtestViewWorkRequestPlugin(work_request)
         context_data = autopkgtest_view.get_context_data()
         self.assertEqual(
             context_data["request_data"]["host_architecture"], "amd64"
@@ -711,7 +714,7 @@ class AutopkgtestViewTests(TestCase):
         work_request.configured_task_data = work_request.task_data.copy()
         work_request.configured_task_data["host_architecture"] = "arm64"
         work_request.save()
-        autopkgtest_view = AutopkgtestView(work_request)
+        autopkgtest_view = AutopkgtestViewWorkRequestPlugin(work_request)
         context_data = autopkgtest_view.get_context_data()
         self.assertEqual(
             context_data["request_data"]["host_architecture"], "arm64"
