@@ -31,6 +31,7 @@ from debusine.test.django import (
     override_permission,
 )
 from debusine.web.views.base import BaseUIView
+from debusine.web.views.places import Place
 from debusine.web.views.tests.utils import ViewTestMixin
 
 
@@ -51,6 +52,7 @@ class TestBaseUI(ViewTestMixin, TestCase):
         request = factory.get("/")
         assert context.user is not None
         request.user = context.user
+        request.session = self.client.session
         view = TemplateBaseUIView.as_view()
         response = cast(TemplateResponse, view(request))
         response.render()
@@ -88,6 +90,7 @@ class TestBaseUI(ViewTestMixin, TestCase):
         """Check if init_view is called for the given method."""
         factory = RequestFactory()
         request = getattr(factory, method)("/")
+        request.session = self.client.session
 
         if add_method:
             view_cls = type(
@@ -126,9 +129,10 @@ class TestBaseUI(ViewTestMixin, TestCase):
         context.set_scope(self.scenario.scope)
         context.set_user(self.scenario.user)
         view = BaseUIView()
+        view.request = RequestFactory().get("/")
         ctx = view.get_context_data()
         self.assertEqual(ctx["base_template"], "web/_base.html")
-        self.assertEqual(ctx["title"], "")
+        self.assertEqual(ctx["place"], Place(title=""))
         self.assertNotIn("workspace_collections", ctx)
 
     def test_get_context_data_with_workspace(self) -> None:
@@ -140,9 +144,10 @@ class TestBaseUI(ViewTestMixin, TestCase):
         )
 
         view = BaseUIView()
+        view.request = RequestFactory().get("/")
         ctx = view.get_context_data()
         self.assertEqual(ctx["base_template"], "web/_base.html")
-        self.assertEqual(ctx["title"], "")
+        self.assertEqual(ctx["place"], Place(title=""))
         self.assertQuerySetEqual(
             ctx["workspace_collections"],
             list(
@@ -158,6 +163,7 @@ class TestBaseUI(ViewTestMixin, TestCase):
         context.set_scope(self.scenario.scope)
         context.set_user(AnonymousUser())
         view = BaseUIView()
+        view.request = RequestFactory().get("/")
 
         with mock.patch(
             "debusine.web.views.base.version", return_value="0.1.0"
@@ -172,6 +178,7 @@ class TestBaseUI(ViewTestMixin, TestCase):
         context.set_scope(self.scenario.scope)
         context.set_user(AnonymousUser())
         view = BaseUIView()
+        view.request = RequestFactory().get("/")
 
         with mock.patch(
             "debusine.web.views.base.version", side_effect=PackageNotFoundError
@@ -271,8 +278,9 @@ class TestBaseUI(ViewTestMixin, TestCase):
         tree = self.assertResponseHTML(response)
 
         self.assertTextContentEqual(
-            tree.xpath("//footer/p")[-1],
-            "Documentation • Bugs • Code • Contributing • 0.1.0",
+            tree.xpath("//footer/div[@id='footer-software']")[0],
+            "Debusine software project: Documentation • Bugs • Code • "
+            "Contributing • 0.1.0",
         )
 
     def test_footer_without_version(self) -> None:
@@ -286,8 +294,9 @@ class TestBaseUI(ViewTestMixin, TestCase):
         tree = self.assertResponseHTML(response)
 
         self.assertTextContentEqual(
-            tree.xpath("//footer/p")[-1],
-            "Documentation • Bugs • Code • Contributing",
+            tree.xpath("//footer/div[@id='footer-software']")[0],
+            "Debusine software project: Documentation • Bugs • Code • "
+            "Contributing",
         )
 
     def test_footer_custom_replace(self) -> None:
@@ -295,9 +304,11 @@ class TestBaseUI(ViewTestMixin, TestCase):
         context.set_scope(self.scenario.scope)
         context.set_user(AnonymousUser())
         tree = self.assertResponseHTML(self.render_base())
-        footer = self.assertHasElement(tree, "//footer")
+        footer = self.assertHasElement(
+            tree, "//footer/*/div[@id='footer-status']"
+        )
         self.assertTextContentEqual(
-            footer.p[0], "Status: Worker pools • Workers • Task status"
+            footer, "Status: Worker pools • Workers • Task status"
         )
 
         with self.custom_template("web/footer.html") as template:
@@ -311,19 +322,37 @@ class TestBaseUI(ViewTestMixin, TestCase):
         context.set_scope(self.scenario.scope)
         context.set_user(AnonymousUser())
         tree = self.assertResponseHTML(self.render_base())
-        footer = self.assertHasElement(tree, "//footer")
+        footer = self.assertHasElement(
+            tree, "//footer/*/div[@id='footer-status']"
+        )
         self.assertTextContentEqual(
-            footer.p[0], "Status: Worker pools • Workers • Task status"
+            footer, "Status: Worker pools • Workers • Task status"
         )
 
         with self.custom_template("web/footer.html") as template:
             template.write_text(
                 "{% extends 'web/footer.html' %}"
-                "{% block status %}{{block.super}} • Testing{% endblock %}"
+                "{% block footer_status %}{{block.super}}"
+                "<div id='footer-testing'>Testing</div>{% endblock %}"
             )
             tree = self.assertResponseHTML(self.render_base())
-            footer = self.assertHasElement(tree, "//footer")
-            self.assertTextContentEqual(
-                footer.p[0],
-                "Status: Worker pools • Workers • Task status • Testing",
+            footer = self.assertHasElement(
+                tree, "//footer/div[@id='footer-utils']"
             )
+            self.assertTextContentEqual(
+                footer,
+                "Status: Worker pools • Workers • Task status"
+                " Testing"
+                " Theme: dark • light • auto",
+            )
+
+    def test_theme_selection(self) -> None:
+        context.set_scope(self.scenario.scope)
+        context.set_user(AnonymousUser())
+        view = BaseUIView()
+        view.request = RequestFactory().get("/")
+        ctx = view.get_context_data()
+        self.assertIsNone(ctx["theme"])
+        view.request.COOKIES["theme"] = "foo"
+        ctx = view.get_context_data()
+        self.assertEqual(ctx["theme"], "foo")
