@@ -188,10 +188,7 @@ class WorkflowTableTests(TestCase):
         self.workflow_1.save()
         self.workflow_2.status = WorkRequest.Statuses.RUNNING
         self.workflow_2.save()
-        self.workflow_1.create_child_worker(
-            "noop",
-            status=WorkRequest.Statuses.BLOCKED,
-        )
+        self.workflow_1.create_child_worker("noop")
         self.assert_filtered(self.workflow_1, statuses=["running__blocked"])
 
     def test_list_filtering_runtime_any(self) -> None:
@@ -202,10 +199,7 @@ class WorkflowTableTests(TestCase):
         """
         self.workflow_1.status = WorkRequest.Statuses.RUNNING
         self.workflow_1.save()
-        self.workflow_1.create_child_worker(
-            "noop",
-            status=WorkRequest.Statuses.BLOCKED,
-        )
+        self.workflow_1.create_child_worker("noop")
         self.assert_filtered(self.workflow_1, statuses=["running__any"])
 
     def test_list_filtering_result(self) -> None:
