@@ -70,7 +70,7 @@ class HomepageViewTests(ViewTestMixin, TestCase):
         self.assertTextContentEqual(h1, "Welcome to Debusine!")
 
         title = self.assertHasElement(tree, "//title")
-        self.assertTextContentEqual(title, "Debusine - Homepage")
+        self.assertTextContentEqual(title, "Debusine - Welcome to Debusine!")
 
         self.assertNavCommonElements(tree)
         self.assertNavNoUser(tree)
@@ -221,7 +221,7 @@ class HomepageViewTests(ViewTestMixin, TestCase):
         with self.custom_template("web/homepage.html") as template:
             template.write_text(
                 "{% extends 'web/homepage.html' %}"
-                "{% block introduction %}"
+                "{% block pagetitle %}"
                 "<h1>Welcome to a custom Debusine</h1>"
                 "{% endblock %}"
             )
@@ -486,3 +486,44 @@ class ToastViewTests(ViewTestMixin, TestCase):
             with self.subTest(type_arg=type_arg):
                 msgs = self.get(type_arg)
                 self.assertEqual([m.level_tag for m in msgs], expected_levels)
+
+
+class ThemeSelectionViewTests(ViewTestMixin, TestCase):
+    """Tests for the :py:class:`ThemeSelectionView` class."""
+
+    def test_get(self) -> None:
+        response = self.client.get(reverse("theme-selection"))
+        self.assertEqual(
+            response.status_code, status.HTTP_405_METHOD_NOT_ALLOWED
+        )
+
+    def test_set(self) -> None:
+        for value in ("light", "dark", "foo"):
+            with self.subTest(value=value):
+                response = self.client.post(
+                    reverse("theme-selection"), data={"theme": value}
+                )
+                self.assertRedirects(response, reverse("homepage:homepage"))
+                self.assertEqual(self.client.cookies["theme"].value, value)
+
+    def test_set_auto(self) -> None:
+        response = self.client.post(
+            reverse("theme-selection"), data={"theme": "foo"}
+        )
+        self.assertRedirects(response, reverse("homepage:homepage"))
+        self.assertEqual(self.client.cookies["theme"].value, "foo")
+        response = self.client.post(
+            reverse("theme-selection"), data={"theme": "auto"}
+        )
+        self.assertRedirects(response, reverse("homepage:homepage"))
+        self.assertEqual(self.client.cookies["theme"].value, "")
+
+    def test_set_redirect(self) -> None:
+        response = self.client.post(
+            reverse("theme-selection"),
+            data={"theme": "foo", "next": "http://example.org"},
+        )
+        self.assertRedirects(
+            response, "http://example.org", fetch_redirect_response=False
+        )
+        self.assertEqual(self.client.cookies["theme"].value, "foo")
