@@ -9,8 +9,9 @@
 
 """debusine WorkRequest view."""
 import abc
+import datetime as dt
 import functools
-from datetime import timedelta
+import itertools
 from typing import Any, cast
 
 from django.contrib import messages
@@ -26,6 +27,7 @@ from rest_framework import status
 from debusine.artifacts.models import TaskTypes
 from debusine.db.context import context
 from debusine.db.models import Artifact, TaskDatabase, User, WorkRequest
+from debusine.db.models.artifacts import ArtifactQuerySet
 from debusine.db.models.work_requests import (
     CannotAbort,
     CannotRetry,
@@ -33,10 +35,15 @@ from debusine.db.models.work_requests import (
     InternalTaskError,
     WorkRequestQuerySet,
 )
+from debusine.server.tasks.wait import Confirm
 from debusine.server.workflows.models import WorkRequestManualUnblockAction
 from debusine.tasks import BaseTask, TaskConfigError
 from debusine.tasks.models import OutputData
-from debusine.web.forms import WorkRequestForm, WorkRequestUnblockForm
+from debusine.web.forms import (
+    WorkRequestConfirmForm,
+    WorkRequestForm,
+    WorkRequestUnblockForm,
+)
 from debusine.web.views import sidebar, ui_shortcuts
 from debusine.web.views.base import (
     CreateViewBase,
@@ -48,8 +55,9 @@ from debusine.web.views.base import (
 )
 from debusine.web.views.http_errors import HttpError400, catch_http_errors
 from debusine.web.views.mixins import RightbarUIMixin, UIShortcutsMixin
+from debusine.web.views.places import Place
 from debusine.web.views.table import TableMixin
-from debusine.web.views.tables import WorkRequestTable
+from debusine.web.views.tables import ArtifactTable, WorkRequestTable
 from debusine.web.views.view_utils import format_json, format_yaml
 
 
@@ -81,7 +89,9 @@ class WorkRequestDetailView(
 ):
     """Show a work request."""
 
-    model = WorkRequest
+    queryset = WorkRequest.objects.select_related(
+        "workspace__scope", "workflow_root"
+    )
     context_object_name = "work_request"
     template_name = "web/work_request-detail.html"
 
@@ -117,7 +127,7 @@ class WorkRequestDetailView(
             items.append(sidebar.create_work_request_superseded(self.object))
         if self.object.supersedes:
             items.append(sidebar.create_work_request_supersedes(self.object))
-        items.append(sidebar.create_workflow(self.object.get_workflow_root()))
+        items.append(sidebar.create_workflow(self.object.workflow_root))
         items.append(sidebar.create_work_request_status(self.object))
         items.append(sidebar.create_workspace(self.object.workspace))
         items.append(
@@ -130,9 +140,9 @@ class WorkRequestDetailView(
         items.append(sidebar.create_expire_at(self.object.expire_at))
         return items
 
-    def get_title(self) -> str:
-        """Get the title for the page."""
-        return f"{self.object.id}: {self.object.get_label()}"
+    def get_place(self) -> Place:
+        """Return the Place for this page."""
+        return self.object.ui(self.request).place
 
     def get_manual_unblock_log(self) -> list[dict[str, Any]]:
         """Get a representation of the manual unblock log."""
@@ -203,31 +213,54 @@ class WorkRequestDetailView(
             task = self.object.get_task()
         except (TaskConfigError, InternalTaskError):
             task = None
-        input_artifacts: list[Artifact] = []
+
+        if self.object.requires_confirmation:
+            assert isinstance(task, Confirm)
+            context["confirm_form"] = WorkRequestConfirmForm()
+            context["confirm_label"] = task.data.confirm_label
+            context["deny_label"] = task.data.deny_label
+
+        input_artifacts: ArtifactQuerySet[Any] = Artifact.objects.none()
         if task and (input_artifact_ids := task.get_input_artifacts_ids()):
-            input_artifacts.extend(
-                Artifact.objects.filter(pk__in=input_artifact_ids)
-                .order_by("id")
-                .annotate_complete()
-            )
+            input_artifacts = Artifact.objects.filter(
+                pk__in=input_artifact_ids
+            ).annotate_complete()
 
-        context["input_artifacts"] = input_artifacts
-
-        built_artifacts = list(
-            Artifact.objects.filter(created_by_work_request=self.object)
-            .annotate_complete()
-            .order_by("category", "id")
+        input_artifacts_table = ArtifactTable(
+            self.request,
+            input_artifacts,
+            prefix="input_artifacts",
+            fragment="work_request",
         )
+        input_artifacts_table_paginator = input_artifacts_table.get_paginator(
+            per_page=20
+        )
+        context["input_artifacts_table"] = input_artifacts_table_paginator
+
+        built_artifacts_table = ArtifactTable(
+            self.request,
+            Artifact.objects.filter(
+                created_by_work_request=self.object
+            ).annotate_complete(),
+            prefix="built_artifacts",
+            fragment="output",
+        )
+        built_artifacts_table_paginator = built_artifacts_table.get_paginator(
+            per_page=20
+        )
+        context["built_artifacts_table"] = built_artifacts_table_paginator
 
         # Generate UI shortcuts for artifacts
-        for artifact in input_artifacts + built_artifacts:
+        for artifact in itertools.chain(
+            input_artifacts_table_paginator.page_obj.object_list,
+            built_artifacts_table_paginator.page_obj.object_list,
+        ):
             self.add_object_ui_shortcuts(
                 artifact,
                 ui_shortcuts.create_artifact_view(artifact),
                 ui_shortcuts.create_artifact_download(artifact),
             )
 
-        context["built_artifacts"] = built_artifacts
         context["task_data"] = format_yaml(self.object.task_data)
 
         if (dynamic_task_data := self.object.dynamic_task_data) is not None:
@@ -306,7 +339,7 @@ class WorkRequestDetailView(
                     value = filesizeformat(value)
 
                 if key in ("duration", "cpu_time") and value is not None:
-                    value = duration_string(timedelta(seconds=value))
+                    value = duration_string(dt.timedelta(seconds=value))
 
                 key = key.replace("_", " ").capitalize().replace("Cpu ", "CPU ")
 
@@ -435,12 +468,11 @@ class WorkRequestListView(
                 continue
 
             if task.host_architecture() == architecture:
-                queryset = (
-                    queryset
-                    | WorkRequest.objects.filter(id=work_request.id).distinct()
+                queryset = queryset | WorkRequest.objects.filter(
+                    id=work_request.id
                 )
 
-        return queryset
+        return queryset.distinct()
 
     def get_queryset(self) -> WorkRequestQuerySet[WorkRequest]:
         """Filter work requests displayed by the workspace GET parameter."""
@@ -576,6 +608,9 @@ class WorkRequestAbortView(
         """Handle POST requests."""
         work_request = self.get_object()
         self.enforce(work_request.can_abort)
+        WorkRequest.objects.filter(
+            id=work_request.id
+        ).lock_workflows_for_update()
         return self.abort(work_request)
 
 
@@ -641,3 +676,74 @@ class WorkRequestUnblockView(
         work_request = self.get_object()
         self.enforce(work_request.can_unblock)
         return self.unblock(work_request)
+
+
+class WorkRequestConfirmView(
+    WorkRequestObjectMixin,
+    SingleObjectMixinBase[WorkRequest],
+    FormMixinBase[WorkRequestConfirmForm],
+    View,
+):
+    """Form view for confirming a work request."""
+
+    model = WorkRequest
+    form_class = WorkRequestConfirmForm
+
+    @property
+    def should_confirm(self) -> bool:
+        """Get whether to confirm the work request from query arguments."""
+        action = self.request.POST["action"]
+        match action:
+            case "confirm":
+                return True
+            case "deny":
+                return False
+            case _:
+                raise HttpError400(f"Invalid action parameter: {action!r}")
+
+    def confirm_or_deny(self, work_request: WorkRequest) -> HttpResponse:
+        """Perform the confirmation or denial."""
+        # This view requires a permission, so the user can't be an
+        # AnonymousUser by this point.
+        assert isinstance(self.request.user, User)
+
+        if (
+            work_request.task_type != TaskTypes.WAIT
+            or work_request.task_name != "confirm"
+        ):
+            raise HttpError400(
+                f"Expected work request to be Wait/Confirm; got "
+                f"{work_request.task_type}/{work_request.task_name}"
+            )
+        task = work_request.get_task()
+        assert isinstance(task, Confirm)
+
+        form = self.get_form()
+        if not form.is_valid():
+            # TODO: This is ugly and we should probably try to render
+            # work_requests:detail with inline confirm form errors instead.
+            # However, since the form is so limited it's very difficult to
+            # actually hit this case in practice, so it doesn't seem worth
+            # the effort for now.
+            return render(
+                self.request,
+                "400.html",
+                context={"error": form.errors.as_json()},
+                status=status.HTTP_400_BAD_REQUEST,
+            )
+
+        task.respond(
+            user=self.request.user,
+            confirm=self.should_confirm,
+            comment=form.cleaned_data["comment"],
+        )
+        return redirect(work_request.get_absolute_url())
+
+    @catch_http_errors
+    def post(
+        self, request: HttpRequest, *args: Any, **kwargs: Any  # noqa: U100
+    ) -> HttpResponse:
+        """Handle POST requests."""
+        work_request = self.get_object()
+        self.enforce(work_request.can_confirm)
+        return self.confirm_or_deny(work_request)
